FHIR_TO_CQL_MAP = {
    "Account": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "status": "String",
        "type": "Concept",
        "name": "String",
        "subject": "List<BackboneElement:Reference>",
        "servicePeriod": "Interval<DateTime>",
        "coverage": "List<BackboneElement:Account_Coverage>",
        "owner": "BackboneElement:Reference",
        "description": "String",
        "guarantor": "List<BackboneElement:Account_Guarantor>",
        "partOf": "BackboneElement:Reference",
    },
    "ActivityDefinition": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "url": "String",
        "identifier": "List<BackboneElement:Identifier>",
        "version": "String",
        "name": "String",
        "title": "String",
        "subtitle": "String",
        "status": "String",
        "experimental": "Boolean",
        "subject": "Choice:String",
        "date": "DateTime",
        "publisher": "String",
        "contact": "List<BackboneElement:ContactDetail>",
        "description": "String",
        "useContext": "List<BackboneElement:UsageContext>",
        "jurisdiction": "List<Concept>",
        "purpose": "String",
        "usage": "String",
        "copyright": "String",
        "approvalDate": "Date",
        "lastReviewDate": "Date",
        "effectivePeriod": "Interval<DateTime>",
        "topic": "List<Concept>",
        "author": "List<BackboneElement:ContactDetail>",
        "editor": "List<BackboneElement:ContactDetail>",
        "reviewer": "List<BackboneElement:ContactDetail>",
        "endorser": "List<BackboneElement:ContactDetail>",
        "relatedArtifact": "List<BackboneElement:RelatedArtifact>",
        "library": "List<String>",
        "kind": "Code",
        "profile": "String",
        "code": "Concept",
        "intent": "Code",
        "priority": "Code",
        "doNotPerform": "Boolean",
        "timing": "Choice:Interval<DateTime>",
        "location": "BackboneElement:Reference",
        "participant": "List<BackboneElement:ActivityDefinition_Participant>",
        "product": "Choice:String",
        "quantity": "Decimal",
        "dosage": "List<BackboneElement:Dosage>",
        "bodySite": "List<Concept>",
        "specimenRequirement": "List<BackboneElement:Reference>",
        "observationRequirement": "List<BackboneElement:Reference>",
        "observationResultRequirement": "List<BackboneElement:Reference>",
        "transform": "String",
        "dynamicValue": "List<BackboneElement:ActivityDefinition_DynamicValue>",
    },
    "AdverseEvent": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "BackboneElement:Identifier",
        "actuality": "String",
        "category": "List<Concept>",
        "event": "Concept",
        "subject": "BackboneElement:Reference",
        "encounter": "BackboneElement:Reference",
        "date": "DateTime",
        "detected": "DateTime",
        "recordedDate": "DateTime",
        "resultingCondition": "List<BackboneElement:Reference>",
        "location": "BackboneElement:Reference",
        "seriousness": "Concept",
        "severity": "Concept",
        "outcome": "Concept",
        "recorder": "BackboneElement:Reference",
        "contributor": "List<BackboneElement:Reference>",
        "suspectEntity": "List<BackboneElement:AdverseEvent_SuspectEntity>",
        "subjectMedicalHistory": "List<BackboneElement:Reference>",
        "referenceDocument": "List<BackboneElement:Reference>",
        "study": "List<BackboneElement:Reference>",
    },
    "AllergyIntolerance": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "clinicalStatus": "Concept",
        "verificationStatus": "Concept",
        "type": "String",
        "category": "List<String>",
        "criticality": "String",
        "code": "Concept",
        "patient": "BackboneElement:Reference",
        "encounter": "BackboneElement:Reference",
        "onset": "Choice:Interval<DateTime>",
        "recordedDate": "DateTime",
        "recorder": "BackboneElement:Reference",
        "asserter": "BackboneElement:Reference",
        "lastOccurrence": "DateTime",
        "note": "List<BackboneElement:Annotation>",
        "reaction": "List<BackboneElement:AllergyIntolerance_Reaction>",
    },
    "Appointment": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "status": "String",
        "cancelationReason": "Concept",
        "serviceCategory": "List<Concept>",
        "serviceType": "List<Concept>",
        "specialty": "List<Concept>",
        "appointmentType": "Concept",
        "reasonCode": "List<Concept>",
        "reasonReference": "List<BackboneElement:Reference>",
        "priority": "String",
        "description": "String",
        "supportingInformation": "List<BackboneElement:Reference>",
        "start": "String",
        "end": "String",
        "minutesDuration": "String",
        "slot": "List<BackboneElement:Reference>",
        "created": "DateTime",
        "comment": "String",
        "patientInstruction": "String",
        "basedOn": "List<BackboneElement:Reference>",
        "participant": "List<BackboneElement:Appointment_Participant>",
        "requestedPeriod": "List<Interval<DateTime>>",
    },
    "AppointmentResponse": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "appointment": "BackboneElement:Reference",
        "start": "String",
        "end": "String",
        "participantType": "List<Concept>",
        "actor": "BackboneElement:Reference",
        "participantStatus": "Code",
        "comment": "String",
    },
    "AuditEvent": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Code",
        "subtype": "List<Code>",
        "action": "String",
        "period": "Interval<DateTime>",
        "recorded": "String",
        "outcome": "String",
        "outcomeDesc": "String",
        "purposeOfEvent": "List<Concept>",
        "agent": "List<BackboneElement:AuditEvent_Agent>",
        "source": "BackboneElement:AuditEvent_Source",
        "entity": "List<BackboneElement:AuditEvent_Entity>",
    },
    "Basic": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "code": "Concept",
        "subject": "BackboneElement:Reference",
        "created": "Date",
        "author": "BackboneElement:Reference",
    },
    "Binary": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "contentType": "Code",
        "securityContext": "BackboneElement:Reference",
        "data": "String",
    },
    "BiologicallyDerivedProduct": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "productCategory": "String",
        "productCode": "Concept",
        "status": "String",
        "request": "List<BackboneElement:Reference>",
        "quantity": "Integer",
        "parent": "List<BackboneElement:Reference>",
        "collection": "BackboneElement:BiologicallyDerivedProduct_Collection",
        "processing": "List<BackboneElement:BiologicallyDerivedProduct_Processing>",
        "manipulation": "BackboneElement:BiologicallyDerivedProduct_Manipulation",
        "storage": "List<BackboneElement:BiologicallyDerivedProduct_Storage>",
    },
    "BodyStructure": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "active": "Boolean",
        "morphology": "Concept",
        "location": "Concept",
        "locationQualifier": "List<Concept>",
        "description": "String",
        "image": "List<BackboneElement:Attachment>",
        "patient": "BackboneElement:Reference",
    },
    "Bundle": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "identifier": "BackboneElement:Identifier",
        "type": "String",
        "timestamp": "String",
        "total": "String",
        "link": "List<BackboneElement:Bundle_Link>",
        "entry": "List<BackboneElement:Bundle_Entry>",
        "signature": "BackboneElement:Signature",
    },
    "CapabilityStatement": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "url": "String",
        "version": "String",
        "name": "String",
        "title": "String",
        "status": "String",
        "experimental": "Boolean",
        "date": "DateTime",
        "publisher": "String",
        "contact": "List<BackboneElement:ContactDetail>",
        "description": "String",
        "useContext": "List<BackboneElement:UsageContext>",
        "jurisdiction": "List<Concept>",
        "purpose": "String",
        "copyright": "String",
        "kind": "String",
        "instantiates": "List<String>",
        "imports": "List<String>",
        "software": "BackboneElement:CapabilityStatement_Software",
        "implementation": "BackboneElement:CapabilityStatement_Implementation",
        "fhirVersion": "String",
        "format": "List<Code>",
        "patchFormat": "List<Code>",
        "implementationGuide": "List<String>",
        "rest": "List<BackboneElement:CapabilityStatement_Rest>",
        "messaging": "List<BackboneElement:CapabilityStatement_Messaging>",
        "document": "List<BackboneElement:CapabilityStatement_Document>",
    },
    "CarePlan": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "instantiatesCanonical": "List<String>",
        "instantiatesUri": "List<String>",
        "basedOn": "List<BackboneElement:Reference>",
        "replaces": "List<BackboneElement:Reference>",
        "partOf": "List<BackboneElement:Reference>",
        "status": "Code",
        "intent": "Code",
        "category": "List<Concept>",
        "title": "String",
        "description": "String",
        "subject": "BackboneElement:Reference",
        "encounter": "BackboneElement:Reference",
        "period": "Interval<DateTime>",
        "created": "DateTime",
        "author": "BackboneElement:Reference",
        "contributor": "List<BackboneElement:Reference>",
        "careTeam": "List<BackboneElement:Reference>",
        "addresses": "List<BackboneElement:Reference>",
        "supportingInfo": "List<BackboneElement:Reference>",
        "goal": "List<BackboneElement:Reference>",
        "activity": "List<BackboneElement:CarePlan_Activity>",
        "note": "List<BackboneElement:Annotation>",
    },
    "CareTeam": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "status": "String",
        "category": "List<Concept>",
        "name": "String",
        "subject": "BackboneElement:Reference",
        "encounter": "BackboneElement:Reference",
        "period": "Interval<DateTime>",
        "participant": "List<BackboneElement:CareTeam_Participant>",
        "reasonCode": "List<Concept>",
        "reasonReference": "List<BackboneElement:Reference>",
        "managingOrganization": "List<BackboneElement:Reference>",
        "telecom": "List<BackboneElement:ContactPoint>",
        "note": "List<BackboneElement:Annotation>",
    },
    "CatalogEntry": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "type": "Concept",
        "orderable": "Boolean",
        "referencedItem": "BackboneElement:Reference",
        "additionalIdentifier": "List<BackboneElement:Identifier>",
        "classification": "List<Concept>",
        "status": "String",
        "validityPeriod": "Interval<DateTime>",
        "validTo": "DateTime",
        "lastUpdated": "DateTime",
        "additionalCharacteristic": "List<Concept>",
        "additionalClassification": "List<Concept>",
        "relatedEntry": "List<BackboneElement:CatalogEntry_RelatedEntry>",
    },
    "ChargeItem": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "definitionUri": "List<String>",
        "definitionCanonical": "List<String>",
        "status": "String",
        "partOf": "List<BackboneElement:Reference>",
        "code": "Concept",
        "subject": "BackboneElement:Reference",
        "context": "BackboneElement:Reference",
        "occurrence": "Choice:Interval<DateTime>",
        "performer": "List<BackboneElement:ChargeItem_Performer>",
        "performingOrganization": "BackboneElement:Reference",
        "requestingOrganization": "BackboneElement:Reference",
        "costCenter": "BackboneElement:Reference",
        "quantity": "Decimal",
        "bodysite": "List<Concept>",
        "factorOverride": "Decimal",
        "priceOverride": "BackboneElement:Money",
        "overrideReason": "String",
        "enterer": "BackboneElement:Reference",
        "enteredDate": "DateTime",
        "reason": "List<Concept>",
        "service": "List<BackboneElement:Reference>",
        "product": "Choice:String",
        "account": "List<BackboneElement:Reference>",
        "note": "List<BackboneElement:Annotation>",
        "supportingInformation": "List<BackboneElement:Reference>",
    },
    "ChargeItemDefinition": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "url": "String",
        "identifier": "List<BackboneElement:Identifier>",
        "version": "String",
        "title": "String",
        "derivedFromUri": "List<String>",
        "partOf": "List<String>",
        "replaces": "List<String>",
        "status": "String",
        "experimental": "Boolean",
        "date": "DateTime",
        "publisher": "String",
        "contact": "List<BackboneElement:ContactDetail>",
        "description": "String",
        "useContext": "List<BackboneElement:UsageContext>",
        "jurisdiction": "List<Concept>",
        "copyright": "String",
        "approvalDate": "Date",
        "lastReviewDate": "Date",
        "effectivePeriod": "Interval<DateTime>",
        "code": "Concept",
        "instance": "List<BackboneElement:Reference>",
        "applicability": "List<BackboneElement:ChargeItemDefinition_Applicability>",
        "propertyGroup": "List<BackboneElement:ChargeItemDefinition_PropertyGroup>",
    },
    "Claim": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "status": "Code",
        "type": "Concept",
        "subType": "Concept",
        "use": "String",
        "patient": "BackboneElement:Reference",
        "billablePeriod": "Interval<DateTime>",
        "created": "DateTime",
        "enterer": "BackboneElement:Reference",
        "insurer": "BackboneElement:Reference",
        "provider": "BackboneElement:Reference",
        "priority": "Concept",
        "fundsReserve": "Concept",
        "related": "List<BackboneElement:Claim_Related>",
        "prescription": "BackboneElement:Reference",
        "originalPrescription": "BackboneElement:Reference",
        "payee": "BackboneElement:Claim_Payee",
        "referral": "BackboneElement:Reference",
        "facility": "BackboneElement:Reference",
        "careTeam": "List<BackboneElement:Claim_CareTeam>",
        "supportingInfo": "List<BackboneElement:Claim_SupportingInfo>",
        "diagnosis": "List<BackboneElement:Claim_Diagnosis>",
        "procedure": "List<BackboneElement:Claim_Procedure>",
        "insurance": "List<BackboneElement:Claim_Insurance>",
        "accident": "BackboneElement:Claim_Accident",
        "item": "List<BackboneElement:Claim_Item>",
        "total": "BackboneElement:Money",
    },
    "ClaimResponse": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "status": "Code",
        "": "Choice:String",
        "use": "Code",
        "patient": "BackboneElement:Reference",
        "created": "DateTime",
        "insurer": "BackboneElement:Reference",
        "requestor": "BackboneElement:Reference",
        "request": "BackboneElement:Reference",
        "outcome": "Code",
        "disposition": "String",
        "preAuthRef": "String",
        "preAuthPeriod": "Interval<DateTime>",
        "payeeType": "Concept",
        "item": "List<BackboneElement:ClaimResponse_Item>",
        "addItem": "List<BackboneElement:ClaimResponse_AddItem>",
        "adjudication": "List<BackboneElement:ClaimResponse_Adjudication>",
        "total": "List<BackboneElement:ClaimResponse_Total>",
        "payment": "BackboneElement:ClaimResponse_Payment",
        "fundsReserve": "Concept",
        "formCode": "Concept",
        "form": "BackboneElement:Attachment",
        "processNote": "List<BackboneElement:ClaimResponse_ProcessNote>",
        "communicationRequest": "List<BackboneElement:Reference>",
        "insurance": "List<BackboneElement:ClaimResponse_Insurance>",
        "error": "List<BackboneElement:ClaimResponse_Error>",
    },
    "ClinicalImpression": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "status": "Code",
        "statusReason": "Concept",
        "code": "Concept",
        "description": "String",
        "subject": "BackboneElement:Reference",
        "encounter": "BackboneElement:Reference",
        "effective": "Choice:Interval<DateTime>",
        "date": "DateTime",
        "assessor": "BackboneElement:Reference",
        "previous": "BackboneElement:Reference",
        "problem": "List<BackboneElement:Reference>",
        "investigation": "List<BackboneElement:ClinicalImpression_Investigation>",
        "protocol": "List<String>",
        "summary": "String",
        "finding": "List<BackboneElement:ClinicalImpression_Finding>",
        "prognosisCodeableConcept": "List<Concept>",
        "prognosisReference": "List<BackboneElement:Reference>",
        "supportingInfo": "List<BackboneElement:Reference>",
        "note": "List<BackboneElement:Annotation>",
    },
    "CodeSystem": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "url": "String",
        "identifier": "List<BackboneElement:Identifier>",
        "version": "String",
        "name": "String",
        "title": "String",
        "status": "String",
        "experimental": "Boolean",
        "date": "DateTime",
        "publisher": "String",
        "contact": "List<BackboneElement:ContactDetail>",
        "description": "String",
        "useContext": "List<BackboneElement:UsageContext>",
        "jurisdiction": "List<Concept>",
        "purpose": "String",
        "copyright": "String",
        "caseSensitive": "Boolean",
        "valueSet": "String",
        "hierarchyMeaning": "String",
        "compositional": "Boolean",
        "versionNeeded": "Boolean",
        "content": "String",
        "supplements": "String",
        "count": "String",
        "filter": "List<BackboneElement:CodeSystem_Filter>",
        "property": "List<BackboneElement:CodeSystem_Property>",
        "concept": "List<BackboneElement:CodeSystem_Concept>",
    },
    "Communication": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "instantiatesCanonical": "List<String>",
        "instantiatesUri": "List<String>",
        "basedOn": "List<BackboneElement:Reference>",
        "partOf": "List<BackboneElement:Reference>",
        "inResponseTo": "List<BackboneElement:Reference>",
        "status": "Code",
        "statusReason": "Concept",
        "category": "List<Concept>",
        "priority": "Code",
        "medium": "List<Concept>",
        "subject": "BackboneElement:Reference",
        "topic": "Concept",
        "about": "List<BackboneElement:Reference>",
        "encounter": "BackboneElement:Reference",
        "sent": "DateTime",
        "received": "DateTime",
        "recipient": "List<BackboneElement:Reference>",
        "sender": "BackboneElement:Reference",
        "reasonCode": "List<Concept>",
        "reasonReference": "List<BackboneElement:Reference>",
        "payload": "List<BackboneElement:Communication_Payload>",
        "note": "List<BackboneElement:Annotation>",
    },
    "CommunicationRequest": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "basedOn": "List<BackboneElement:Reference>",
        "replaces": "List<BackboneElement:Reference>",
        "groupIdentifier": "BackboneElement:Identifier",
        "status": "Code",
        "statusReason": "Concept",
        "category": "List<Concept>",
        "priority": "Code",
        "doNotPerform": "Boolean",
        "medium": "List<Concept>",
        "subject": "BackboneElement:Reference",
        "about": "List<BackboneElement:Reference>",
        "encounter": "BackboneElement:Reference",
        "payload": "List<BackboneElement:CommunicationRequest_Payload>",
        "occurrence": "Choice:Interval<DateTime>",
        "authoredOn": "DateTime",
        "requester": "BackboneElement:Reference",
        "recipient": "List<BackboneElement:Reference>",
        "sender": "BackboneElement:Reference",
        "reasonCode": "List<Concept>",
        "reasonReference": "List<BackboneElement:Reference>",
        "note": "List<BackboneElement:Annotation>",
    },
    "CompartmentDefinition": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "url": "String",
        "version": "String",
        "name": "String",
        "status": "String",
        "experimental": "Boolean",
        "date": "DateTime",
        "publisher": "String",
        "contact": "List<BackboneElement:ContactDetail>",
        "description": "String",
        "useContext": "List<BackboneElement:UsageContext>",
        "purpose": "String",
        "code": "String",
        "search": "Boolean",
        "resource": "List<BackboneElement:CompartmentDefinition_Resource>",
    },
    "Composition": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "BackboneElement:Identifier",
        "status": "String",
        "type": "Concept",
        "category": "List<Concept>",
        "subject": "BackboneElement:Reference",
        "encounter": "BackboneElement:Reference",
        "date": "DateTime",
        "author": "List<BackboneElement:Reference>",
        "title": "String",
        "confidentiality": "Code",
        "attester": "List<BackboneElement:Composition_Attester>",
        "custodian": "BackboneElement:Reference",
        "relatesTo": "List<BackboneElement:Composition_RelatesTo>",
        "event": "List<BackboneElement:Composition_Event>",
        "section": "List<BackboneElement:Composition_Section>",
    },
    "ConceptMap": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "url": "String",
        "identifier": "BackboneElement:Identifier",
        "version": "String",
        "name": "String",
        "title": "String",
        "status": "String",
        "experimental": "Boolean",
        "date": "DateTime",
        "publisher": "String",
        "contact": "List<BackboneElement:ContactDetail>",
        "description": "String",
        "useContext": "List<BackboneElement:UsageContext>",
        "jurisdiction": "List<Concept>",
        "purpose": "String",
        "copyright": "String",
        "source": "Choice:String",
        "target": "Choice:String",
        "group": "List<BackboneElement:ConceptMap_Group>",
    },
    "Condition": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "clinicalStatus": "Concept",
        "verificationStatus": "Concept",
        "category": "List<Concept>",
        "severity": "Concept",
        "code": "Concept",
        "bodySite": "List<Concept>",
        "subject": "BackboneElement:Reference",
        "encounter": "BackboneElement:Reference",
        "onset": "Choice:Interval<DateTime>",
        "abatement": "Choice:Interval<DateTime>",
        "recordedDate": "DateTime",
        "recorder": "BackboneElement:Reference",
        "asserter": "BackboneElement:Reference",
        "stage": "List<BackboneElement:Condition_Stage>",
        "evidence": "List<BackboneElement:Condition_Evidence>",
        "note": "List<BackboneElement:Annotation>",
    },
    "Consent": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "status": "String",
        "scope": "Concept",
        "category": "List<Concept>",
        "patient": "BackboneElement:Reference",
        "dateTime": "DateTime",
        "performer": "List<BackboneElement:Reference>",
        "organization": "List<BackboneElement:Reference>",
        "source": "Choice:String",
        "policy": "List<BackboneElement:Consent_Policy>",
        "policyRule": "Concept",
        "verification": "List<BackboneElement:Consent_Verification>",
        "provision": "BackboneElement:Consent_Provision",
    },
    "Contract": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "url": "String",
        "version": "String",
        "status": "Code",
        "legalState": "Concept",
        "instantiatesCanonical": "BackboneElement:Reference",
        "instantiatesUri": "String",
        "contentDerivative": "Concept",
        "issued": "DateTime",
        "applies": "Interval<DateTime>",
        "expirationType": "Concept",
        "subject": "List<BackboneElement:Reference>",
        "authority": "List<BackboneElement:Reference>",
        "domain": "List<BackboneElement:Reference>",
        "site": "List<BackboneElement:Reference>",
        "name": "String",
        "title": "String",
        "subtitle": "String",
        "alias": "List<String>",
        "author": "BackboneElement:Reference",
        "scope": "Concept",
        "topic": "Choice:String",
        "type": "Concept",
        "subType": "List<Concept>",
        "contentDefinition": "BackboneElement:Contract_ContentDefinition",
        "term": "List<BackboneElement:Contract_Term>",
        "supportingInfo": "List<BackboneElement:Reference>",
        "relevantHistory": "List<BackboneElement:Reference>",
        "signer": "List<BackboneElement:Contract_Signer>",
        "friendly": "List<BackboneElement:Contract_Friendly>",
        "legal": "List<BackboneElement:Contract_Legal>",
        "rule": "List<BackboneElement:Contract_Rule>",
        "legallyBinding": "Choice:String",
    },
    "Coverage": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "status": "Code",
        "type": "Concept",
        "policyHolder": "BackboneElement:Reference",
        "subscriber": "BackboneElement:Reference",
        "subscriberId": "String",
        "beneficiary": "BackboneElement:Reference",
        "dependent": "String",
        "relationship": "Concept",
        "period": "Interval<DateTime>",
        "payor": "List<BackboneElement:Reference>",
        "class": "List<BackboneElement:Coverage_Class>",
        "order": "String",
        "network": "String",
        "costToBeneficiary": "List<BackboneElement:Coverage_CostToBeneficiary>",
        "subrogation": "Boolean",
        "contract": "List<BackboneElement:Reference>",
    },
    "CoverageEligibilityRequest": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "status": "Code",
        "priority": "Concept",
        "purpose": "List<String>",
        "patient": "BackboneElement:Reference",
        "serviced": "Choice:Interval<DateTime>",
        "created": "DateTime",
        "enterer": "BackboneElement:Reference",
        "provider": "BackboneElement:Reference",
        "insurer": "BackboneElement:Reference",
        "facility": "BackboneElement:Reference",
        "supportingInfo": "List<BackboneElement:CoverageEligibilityRequest_SupportingInfo>",
        "insurance": "List<BackboneElement:CoverageEligibilityRequest_Insurance>",
        "item": "List<BackboneElement:CoverageEligibilityRequest_Item>",
    },
    "CoverageEligibilityResponse": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "status": "Code",
        "purpose": "List<String>",
        "patient": "BackboneElement:Reference",
        "serviced": "Choice:Interval<DateTime>",
        "created": "DateTime",
        "requestor": "BackboneElement:Reference",
        "request": "BackboneElement:Reference",
        "outcome": "String",
        "disposition": "String",
        "insurer": "BackboneElement:Reference",
        "insurance": "List<BackboneElement:CoverageEligibilityResponse_Insurance>",
        "preAuthRef": "String",
        "form": "Concept",
        "error": "List<BackboneElement:CoverageEligibilityResponse_Error>",
    },
    "DetectedIssue": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "status": "Code",
        "code": "Concept",
        "severity": "String",
        "patient": "BackboneElement:Reference",
        "identified": "Choice:Interval<DateTime>",
        "author": "BackboneElement:Reference",
        "implicated": "List<BackboneElement:Reference>",
        "evidence": "List<BackboneElement:DetectedIssue_Evidence>",
        "detail": "String",
        "reference": "String",
        "mitigation": "List<BackboneElement:DetectedIssue_Mitigation>",
    },
    "Device": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "definition": "BackboneElement:Reference",
        "udiCarrier": "List<BackboneElement:Device_UdiCarrier>",
        "status": "String",
        "statusReason": "List<Concept>",
        "distinctIdentifier": "String",
        "manufacturer": "String",
        "manufactureDate": "DateTime",
        "expirationDate": "DateTime",
        "lotNumber": "String",
        "serialNumber": "String",
        "deviceName": "List<BackboneElement:Device_DeviceName>",
        "modelNumber": "String",
        "partNumber": "String",
        "type": "Concept",
        "specialization": "List<BackboneElement:Device_Specialization>",
        "version": "List<BackboneElement:Device_Version>",
        "property": "List<BackboneElement:Device_Property>",
        "patient": "BackboneElement:Reference",
        "owner": "BackboneElement:Reference",
        "contact": "List<BackboneElement:ContactPoint>",
        "location": "BackboneElement:Reference",
        "url": "String",
        "note": "List<BackboneElement:Annotation>",
        "safety": "List<Concept>",
        "parent": "BackboneElement:Reference",
    },
    "DeviceDefinition": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "udiDeviceIdentifier": "List<BackboneElement:DeviceDefinition_UdiDeviceIdentifier>",
        "manufacturer": "Choice:String",
        "deviceName": "List<BackboneElement:DeviceDefinition_DeviceName>",
        "modelNumber": "String",
        "type": "Concept",
        "specialization": "List<BackboneElement:DeviceDefinition_Specialization>",
        "version": "List<String>",
        "safety": "List<Concept>",
        "shelfLifeStorage": "List<BackboneElement:ProductShelfLife>",
        "physicalCharacteristics": "BackboneElement:ProdCharacteristic",
        "languageCode": "List<Concept>",
        "capability": "List<BackboneElement:DeviceDefinition_Capability>",
        "property": "List<BackboneElement:DeviceDefinition_Property>",
        "owner": "BackboneElement:Reference",
        "contact": "List<BackboneElement:ContactPoint>",
        "url": "String",
        "onlineInformation": "String",
        "note": "List<BackboneElement:Annotation>",
        "quantity": "Decimal",
        "parentDevice": "BackboneElement:Reference",
        "material": "List<BackboneElement:DeviceDefinition_Material>",
    },
    "DeviceMetric": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "type": "Concept",
        "unit": "Concept",
        "source": "BackboneElement:Reference",
        "parent": "BackboneElement:Reference",
        "operationalStatus": "String",
        "color": "String",
        "category": "String",
        "measurementPeriod": "BackboneElement:Timing",
        "calibration": "List<BackboneElement:DeviceMetric_Calibration>",
    },
    "DeviceRequest": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "instantiatesCanonical": "List<String>",
        "instantiatesUri": "List<String>",
        "basedOn": "List<BackboneElement:Reference>",
        "priorRequest": "List<BackboneElement:Reference>",
        "groupIdentifier": "BackboneElement:Identifier",
        "status": "Code",
        "intent": "Code",
        "priority": "Code",
        "code": "Choice:String",
        "parameter": "List<BackboneElement:DeviceRequest_Parameter>",
        "subject": "BackboneElement:Reference",
        "encounter": "BackboneElement:Reference",
        "occurrence": "Choice:Interval<DateTime>",
        "authoredOn": "DateTime",
        "requester": "BackboneElement:Reference",
        "performerType": "Concept",
        "performer": "BackboneElement:Reference",
        "reason": "Choice:String",
        "insurance": "List<BackboneElement:Reference>",
        "supportingInfo": "List<BackboneElement:Reference>",
        "note": "List<BackboneElement:Annotation>",
        "relevantHistory": "List<BackboneElement:Reference>",
    },
    "DeviceUseStatement": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "basedOn": "List<BackboneElement:Reference>",
        "status": "String",
        "subject": "BackboneElement:Reference",
        "derivedFrom": "List<BackboneElement:Reference>",
        "timing": "Choice:Interval<DateTime>",
        "recordedOn": "DateTime",
        "source": "BackboneElement:Reference",
        "device": "BackboneElement:Reference",
        "reasonCode": "List<Concept>",
        "reasonReference": "List<BackboneElement:Reference>",
        "bodySite": "Concept",
        "note": "List<BackboneElement:Annotation>",
    },
    "DiagnosticReport": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "basedOn": "List<BackboneElement:Reference>",
        "status": "String",
        "category": "List<Concept>",
        "code": "Concept",
        "subject": "BackboneElement:Reference",
        "encounter": "BackboneElement:Reference",
        "effective": "Choice:Interval<DateTime>",
        "issued": "String",
        "performer": "List<BackboneElement:Reference>",
        "resultsInterpreter": "List<BackboneElement:Reference>",
        "specimen": "List<BackboneElement:Reference>",
        "result": "List<BackboneElement:Reference>",
        "imagingStudy": "List<BackboneElement:Reference>",
        "media": "List<BackboneElement:DiagnosticReport_Media>",
        "conclusion": "String",
        "conclusionCode": "List<Concept>",
        "presentedForm": "List<BackboneElement:Attachment>",
    },
    "DocumentManifest": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "masterIdentifier": "BackboneElement:Identifier",
        "identifier": "List<BackboneElement:Identifier>",
        "status": "String",
        "type": "Concept",
        "subject": "BackboneElement:Reference",
        "created": "DateTime",
        "author": "List<BackboneElement:Reference>",
        "recipient": "List<BackboneElement:Reference>",
        "source": "String",
        "description": "String",
        "content": "List<BackboneElement:Reference>",
        "related": "List<BackboneElement:DocumentManifest_Related>",
    },
    "DocumentReference": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "masterIdentifier": "BackboneElement:Identifier",
        "identifier": "List<BackboneElement:Identifier>",
        "status": "String",
        "docStatus": "Code",
        "type": "Concept",
        "category": "List<Concept>",
        "subject": "BackboneElement:Reference",
        "date": "String",
        "author": "List<BackboneElement:Reference>",
        "authenticator": "BackboneElement:Reference",
        "custodian": "BackboneElement:Reference",
        "relatesTo": "List<BackboneElement:DocumentReference_RelatesTo>",
        "description": "String",
        "securityLabel": "List<Concept>",
        "content": "List<BackboneElement:DocumentReference_Content>",
        "context": "BackboneElement:DocumentReference_Context",
    },
    "EffectEvidenceSynthesis": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "url": "String",
        "identifier": "List<BackboneElement:Identifier>",
        "version": "String",
        "name": "String",
        "title": "String",
        "status": "String",
        "date": "DateTime",
        "publisher": "String",
        "contact": "List<BackboneElement:ContactDetail>",
        "description": "String",
        "note": "List<BackboneElement:Annotation>",
        "useContext": "List<BackboneElement:UsageContext>",
        "jurisdiction": "List<Concept>",
        "copyright": "String",
        "approvalDate": "Date",
        "lastReviewDate": "Date",
        "effectivePeriod": "Interval<DateTime>",
        "topic": "List<Concept>",
        "author": "List<BackboneElement:ContactDetail>",
        "editor": "List<BackboneElement:ContactDetail>",
        "reviewer": "List<BackboneElement:ContactDetail>",
        "endorser": "List<BackboneElement:ContactDetail>",
        "relatedArtifact": "List<BackboneElement:RelatedArtifact>",
        "synthesisType": "Concept",
        "studyType": "Concept",
        "population": "BackboneElement:Reference",
        "exposure": "BackboneElement:Reference",
        "exposureAlternative": "BackboneElement:Reference",
        "outcome": "BackboneElement:Reference",
        "sampleSize": "BackboneElement:EffectEvidenceSynthesis_SampleSize",
        "resultsByExposure": "List<BackboneElement:EffectEvidenceSynthesis_ResultsByExposure>",
        "effectEstimate": "List<BackboneElement:EffectEvidenceSynthesis_EffectEstimate>",
        "certainty": "List<BackboneElement:EffectEvidenceSynthesis_Certainty>",
    },
    "Encounter": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "status": "String",
        "statusHistory": "List<BackboneElement:Encounter_StatusHistory>",
        "class": "Code",
        "classHistory": "List<BackboneElement:Encounter_ClassHistory>",
        "type": "List<Concept>",
        "serviceType": "Concept",
        "priority": "Concept",
        "subject": "BackboneElement:Reference",
        "episodeOfCare": "List<BackboneElement:Reference>",
        "basedOn": "List<BackboneElement:Reference>",
        "participant": "List<BackboneElement:Encounter_Participant>",
        "appointment": "List<BackboneElement:Reference>",
        "period": "Interval<DateTime>",
        "length": "BackboneElement:Duration",
        "reason": "Choice:String",
        "diagnosis": "List<BackboneElement:Encounter_Diagnosis>",
        "account": "List<BackboneElement:Reference>",
        "hospitalization": "BackboneElement:Encounter_Hospitalization",
        "location": "List<BackboneElement:Encounter_Location>",
        "serviceProvider": "BackboneElement:Reference",
        "partOf": "BackboneElement:Reference",
    },
    "Endpoint": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "status": "String",
        "connectionType": "Code",
        "name": "String",
        "managingOrganization": "BackboneElement:Reference",
        "contact": "List<BackboneElement:ContactPoint>",
        "period": "Interval<DateTime>",
        "payloadType": "List<Concept>",
        "payloadMimeType": "List<Code>",
        "address": "String",
        "header": "List<String>",
    },
    "EnrollmentRequest": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "status": "Code",
        "created": "DateTime",
        "insurer": "BackboneElement:Reference",
        "provider": "BackboneElement:Reference",
        "candidate": "BackboneElement:Reference",
        "coverage": "BackboneElement:Reference",
    },
    "EnrollmentResponse": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "status": "Code",
        "request": "BackboneElement:Reference",
        "outcome": "String",
        "disposition": "String",
        "created": "DateTime",
        "organization": "BackboneElement:Reference",
        "requestProvider": "BackboneElement:Reference",
    },
    "EpisodeOfCare": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "status": "String",
        "statusHistory": "List<BackboneElement:EpisodeOfCare_StatusHistory>",
        "type": "List<Concept>",
        "diagnosis": "List<BackboneElement:EpisodeOfCare_Diagnosis>",
        "patient": "BackboneElement:Reference",
        "managingOrganization": "BackboneElement:Reference",
        "period": "Interval<DateTime>",
        "referralRequest": "List<BackboneElement:Reference>",
        "careManager": "BackboneElement:Reference",
        "team": "List<BackboneElement:Reference>",
        "account": "List<BackboneElement:Reference>",
    },
    "EventDefinition": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "url": "String",
        "identifier": "List<BackboneElement:Identifier>",
        "version": "String",
        "name": "String",
        "title": "String",
        "subtitle": "String",
        "status": "String",
        "experimental": "Boolean",
        "subject": "Choice:String",
        "date": "DateTime",
        "publisher": "String",
        "contact": "List<BackboneElement:ContactDetail>",
        "description": "String",
        "useContext": "List<BackboneElement:UsageContext>",
        "jurisdiction": "List<Concept>",
        "purpose": "String",
        "usage": "String",
        "copyright": "String",
        "approvalDate": "Date",
        "lastReviewDate": "Date",
        "effectivePeriod": "Interval<DateTime>",
        "topic": "List<Concept>",
        "author": "List<BackboneElement:ContactDetail>",
        "editor": "List<BackboneElement:ContactDetail>",
        "reviewer": "List<BackboneElement:ContactDetail>",
        "endorser": "List<BackboneElement:ContactDetail>",
        "relatedArtifact": "List<BackboneElement:RelatedArtifact>",
        "trigger": "List<BackboneElement:TriggerDefinition>",
    },
    "Evidence": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "url": "String",
        "identifier": "List<BackboneElement:Identifier>",
        "version": "String",
        "name": "String",
        "title": "String",
        "shortTitle": "String",
        "subtitle": "String",
        "status": "String",
        "date": "DateTime",
        "publisher": "String",
        "contact": "List<BackboneElement:ContactDetail>",
        "description": "String",
        "note": "List<BackboneElement:Annotation>",
        "useContext": "List<BackboneElement:UsageContext>",
        "jurisdiction": "List<Concept>",
        "copyright": "String",
        "approvalDate": "Date",
        "lastReviewDate": "Date",
        "effectivePeriod": "Interval<DateTime>",
        "topic": "List<Concept>",
        "author": "List<BackboneElement:ContactDetail>",
        "editor": "List<BackboneElement:ContactDetail>",
        "reviewer": "List<BackboneElement:ContactDetail>",
        "endorser": "List<BackboneElement:ContactDetail>",
        "relatedArtifact": "List<BackboneElement:RelatedArtifact>",
        "exposureBackground": "BackboneElement:Reference",
        "exposureVariant": "List<BackboneElement:Reference>",
        "outcome": "List<BackboneElement:Reference>",
    },
    "EvidenceVariable": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "url": "String",
        "identifier": "List<BackboneElement:Identifier>",
        "version": "String",
        "name": "String",
        "title": "String",
        "shortTitle": "String",
        "subtitle": "String",
        "status": "String",
        "date": "DateTime",
        "publisher": "String",
        "contact": "List<BackboneElement:ContactDetail>",
        "description": "String",
        "note": "List<BackboneElement:Annotation>",
        "useContext": "List<BackboneElement:UsageContext>",
        "jurisdiction": "List<Concept>",
        "copyright": "String",
        "approvalDate": "Date",
        "lastReviewDate": "Date",
        "effectivePeriod": "Interval<DateTime>",
        "topic": "List<Concept>",
        "author": "List<BackboneElement:ContactDetail>",
        "editor": "List<BackboneElement:ContactDetail>",
        "reviewer": "List<BackboneElement:ContactDetail>",
        "endorser": "List<BackboneElement:ContactDetail>",
        "relatedArtifact": "List<BackboneElement:RelatedArtifact>",
        "type": "String",
        "characteristic": "List<BackboneElement:EvidenceVariable_Characteristic>",
    },
    "ExampleScenario": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "url": "String",
        "identifier": "List<BackboneElement:Identifier>",
        "version": "String",
        "name": "String",
        "status": "String",
        "experimental": "Boolean",
        "date": "DateTime",
        "publisher": "String",
        "contact": "List<BackboneElement:ContactDetail>",
        "useContext": "List<BackboneElement:UsageContext>",
        "jurisdiction": "List<Concept>",
        "copyright": "String",
        "purpose": "String",
        "actor": "List<BackboneElement:ExampleScenario_Actor>",
        "instance": "List<BackboneElement:ExampleScenario_Instance>",
        "process": "List<BackboneElement:ExampleScenario_Process>",
        "workflow": "List<String>",
    },
    "ExplanationOfBenefit": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "status": "String",
        "type": "Concept",
        "subType": "Concept",
        "use": "Code",
        "patient": "BackboneElement:Reference",
        "billablePeriod": "Interval<DateTime>",
        "created": "DateTime",
        "enterer": "BackboneElement:Reference",
        "insurer": "BackboneElement:Reference",
        "provider": "BackboneElement:Reference",
        "priority": "Concept",
        "fundsReserveRequested": "Concept",
        "fundsReserve": "Concept",
        "related": "List<BackboneElement:ExplanationOfBenefit_Related>",
        "prescription": "BackboneElement:Reference",
        "originalPrescription": "BackboneElement:Reference",
        "payee": "BackboneElement:ExplanationOfBenefit_Payee",
        "referral": "BackboneElement:Reference",
        "facility": "BackboneElement:Reference",
        "claim": "BackboneElement:Reference",
        "claimResponse": "BackboneElement:Reference",
        "outcome": "Code",
        "disposition": "String",
        "preAuthRef": "List<String>",
        "preAuthRefPeriod": "List<Interval<DateTime>>",
        "careTeam": "List<BackboneElement:ExplanationOfBenefit_CareTeam>",
        "supportingInfo": "List<BackboneElement:ExplanationOfBenefit_SupportingInfo>",
        "diagnosis": "List<BackboneElement:ExplanationOfBenefit_Diagnosis>",
        "procedure": "List<BackboneElement:ExplanationOfBenefit_Procedure>",
        "precedence": "String",
        "insurance": "List<BackboneElement:ExplanationOfBenefit_Insurance>",
        "accident": "BackboneElement:ExplanationOfBenefit_Accident",
        "item": "List<BackboneElement:ExplanationOfBenefit_Item>",
        "addItem": "List<BackboneElement:ExplanationOfBenefit_AddItem>",
        "adjudication": "List<BackboneElement:ExplanationOfBenefit_Adjudication>",
        "total": "List<BackboneElement:ExplanationOfBenefit_Total>",
        "payment": "BackboneElement:ExplanationOfBenefit_Payment",
        "formCode": "Concept",
        "form": "BackboneElement:Attachment",
        "processNote": "List<BackboneElement:ExplanationOfBenefit_ProcessNote>",
        "benefitPeriod": "Interval<DateTime>",
        "benefitBalance": "List<BackboneElement:ExplanationOfBenefit_BenefitBalance>",
    },
    "FamilyMemberHistory": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "instantiatesCanonical": "List<String>",
        "instantiatesUri": "List<String>",
        "status": "String",
        "dataAbsentReason": "Concept",
        "patient": "BackboneElement:Reference",
        "date": "DateTime",
        "name": "String",
        "relationship": "Concept",
        "sex": "Concept",
        "born": "Choice:Interval<DateTime>",
        "age": "Choice:String",
        "estimatedAge": "Boolean",
        "deceased": "Choice:String",
        "reasonCode": "List<Concept>",
        "reasonReference": "List<BackboneElement:Reference>",
        "note": "List<BackboneElement:Annotation>",
        "condition": "List<BackboneElement:FamilyMemberHistory_Condition>",
    },
    "Flag": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "status": "String",
        "category": "List<Concept>",
        "code": "Concept",
        "subject": "BackboneElement:Reference",
        "period": "Interval<DateTime>",
        "encounter": "BackboneElement:Reference",
        "author": "BackboneElement:Reference",
    },
    "Goal": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "lifecycleStatus": "String",
        "achievementStatus": "Concept",
        "category": "List<Concept>",
        "priority": "Concept",
        "description": "Concept",
        "subject": "BackboneElement:Reference",
        "start": "Choice:String",
        "target": "List<BackboneElement:Goal_Target>",
        "statusDate": "Date",
        "statusReason": "String",
        "expressedBy": "BackboneElement:Reference",
        "addresses": "List<BackboneElement:Reference>",
        "note": "List<BackboneElement:Annotation>",
        "outcomeCode": "List<Concept>",
        "outcomeReference": "List<BackboneElement:Reference>",
    },
    "GraphDefinition": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "url": "String",
        "version": "String",
        "name": "String",
        "status": "String",
        "experimental": "Boolean",
        "date": "DateTime",
        "publisher": "String",
        "contact": "List<BackboneElement:ContactDetail>",
        "description": "String",
        "useContext": "List<BackboneElement:UsageContext>",
        "jurisdiction": "List<Concept>",
        "purpose": "String",
        "start": "Code",
        "profile": "String",
        "link": "List<BackboneElement:GraphDefinition_Link>",
    },
    "Group": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "active": "Boolean",
        "type": "String",
        "actual": "Boolean",
        "code": "Concept",
        "name": "String",
        "quantity": "String",
        "managingEntity": "BackboneElement:Reference",
        "characteristic": "List<BackboneElement:Group_Characteristic>",
        "member": "List<BackboneElement:Group_Member>",
    },
    "GuidanceResponse": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "requestIdentifier": "BackboneElement:Identifier",
        "identifier": "List<BackboneElement:Identifier>",
        "module": "Choice:String",
        "status": "String",
        "subject": "BackboneElement:Reference",
        "encounter": "BackboneElement:Reference",
        "occurrenceDateTime": "DateTime",
        "performer": "BackboneElement:Reference",
        "reasonCode": "List<Concept>",
        "reasonReference": "List<BackboneElement:Reference>",
        "note": "List<BackboneElement:Annotation>",
        "evaluationMessage": "List<BackboneElement:Reference>",
        "outputParameters": "BackboneElement:Reference",
        "result": "BackboneElement:Reference",
        "dataRequirement": "List<BackboneElement:DataRequirement>",
    },
    "HealthcareService": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "active": "Boolean",
        "providedBy": "BackboneElement:Reference",
        "category": "List<Concept>",
        "type": "List<Concept>",
        "specialty": "List<Concept>",
        "location": "List<BackboneElement:Reference>",
        "name": "String",
        "comment": "String",
        "extraDetails": "String",
        "photo": "BackboneElement:Attachment",
        "telecom": "List<BackboneElement:ContactPoint>",
        "coverageArea": "List<BackboneElement:Reference>",
        "serviceProvisionCode": "List<Concept>",
        "eligibility": "List<BackboneElement:HealthcareService_Eligibility>",
        "program": "List<Concept>",
        "characteristic": "List<Concept>",
        "communication": "List<Concept>",
        "referralMethod": "List<Concept>",
        "appointmentRequired": "Boolean",
        "availableTime": "List<BackboneElement:HealthcareService_AvailableTime>",
        "notAvailable": "List<BackboneElement:HealthcareService_NotAvailable>",
        "availabilityExceptions": "String",
        "endpoint": "List<BackboneElement:Reference>",
    },
    "ImagingStudy": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "status": "String",
        "modality": "List<Code>",
        "subject": "BackboneElement:Reference",
        "encounter": "BackboneElement:Reference",
        "started": "DateTime",
        "basedOn": "List<BackboneElement:Reference>",
        "referrer": "BackboneElement:Reference",
        "interpreter": "List<BackboneElement:Reference>",
        "endpoint": "List<BackboneElement:Reference>",
        "numberOfSeries": "String",
        "numberOfInstances": "String",
        "procedureReference": "BackboneElement:Reference",
        "procedureCode": "List<Concept>",
        "location": "BackboneElement:Reference",
        "reasonCode": "List<Concept>",
        "reasonReference": "List<BackboneElement:Reference>",
        "note": "List<BackboneElement:Annotation>",
        "description": "String",
        "series": "List<BackboneElement:ImagingStudy_Series>",
    },
    "Immunization": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "status": "Code",
        "statusReason": "Concept",
        "vaccineCode": "Concept",
        "patient": "BackboneElement:Reference",
        "encounter": "BackboneElement:Reference",
        "occurrence": "Choice:String",
        "recorded": "DateTime",
        "primarySource": "Boolean",
        "reportOrigin": "Concept",
        "location": "BackboneElement:Reference",
        "manufacturer": "BackboneElement:Reference",
        "lotNumber": "String",
        "expirationDate": "Date",
        "site": "Concept",
        "route": "Concept",
        "doseQuantity": "Decimal",
        "performer": "List<BackboneElement:Immunization_Performer>",
        "note": "List<BackboneElement:Annotation>",
        "reasonCode": "List<Concept>",
        "reasonReference": "List<BackboneElement:Reference>",
        "isSubpotent": "Boolean",
        "subpotentReason": "List<Concept>",
        "education": "List<BackboneElement:Immunization_Education>",
        "programEligibility": "List<Concept>",
        "fundingSource": "Concept",
        "reaction": "List<BackboneElement:Immunization_Reaction>",
        "protocolApplied": "List<BackboneElement:Immunization_ProtocolApplied>",
    },
    "ImmunizationEvaluation": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "status": "Code",
        "patient": "BackboneElement:Reference",
        "date": "DateTime",
        "authority": "BackboneElement:Reference",
        "targetDisease": "Concept",
        "immunizationEvent": "BackboneElement:Reference",
        "doseStatus": "Concept",
        "doseStatusReason": "List<Concept>",
        "description": "String",
        "series": "String",
        "doseNumber": "Choice:String",
        "seriesDoses": "Choice:String",
    },
    "ImmunizationRecommendation": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "patient": "BackboneElement:Reference",
        "date": "DateTime",
        "authority": "BackboneElement:Reference",
        "recommendation": "List<BackboneElement:ImmunizationRecommendation_Recommendation>",
    },
    "ImplementationGuide": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "url": "String",
        "version": "String",
        "name": "String",
        "title": "String",
        "status": "String",
        "experimental": "Boolean",
        "date": "DateTime",
        "publisher": "String",
        "contact": "List<BackboneElement:ContactDetail>",
        "description": "String",
        "useContext": "List<BackboneElement:UsageContext>",
        "jurisdiction": "List<Concept>",
        "copyright": "String",
        "packageId": "String",
        "license": "String",
        "fhirVersion": "List<String>",
        "dependsOn": "List<BackboneElement:ImplementationGuide_DependsOn>",
        "global": "List<BackboneElement:ImplementationGuide_Global>",
        "definition": "BackboneElement:ImplementationGuide_Definition",
        "manifest": "BackboneElement:ImplementationGuide_Manifest",
    },
    "InsurancePlan": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "status": "String",
        "type": "List<Concept>",
        "name": "String",
        "alias": "List<String>",
        "period": "Interval<DateTime>",
        "ownedBy": "BackboneElement:Reference",
        "administeredBy": "BackboneElement:Reference",
        "coverageArea": "List<BackboneElement:Reference>",
        "contact": "List<BackboneElement:InsurancePlan_Contact>",
        "endpoint": "List<BackboneElement:Reference>",
        "network": "List<BackboneElement:Reference>",
        "coverage": "List<BackboneElement:InsurancePlan_Coverage>",
        "plan": "List<BackboneElement:InsurancePlan_Plan>",
    },
    "Invoice": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "status": "String",
        "cancelledReason": "String",
        "type": "Concept",
        "subject": "BackboneElement:Reference",
        "recipient": "BackboneElement:Reference",
        "date": "DateTime",
        "participant": "List<BackboneElement:Invoice_Participant>",
        "issuer": "BackboneElement:Reference",
        "account": "BackboneElement:Reference",
        "lineItem": "List<BackboneElement:Invoice_LineItem>",
        "totalPriceComponent": "List<BackboneElement:Invoice_PriceComponent>",
        "totalNet": "BackboneElement:Money",
        "totalGross": "BackboneElement:Money",
        "paymentTerms": "String",
        "note": "List<BackboneElement:Annotation>",
    },
    "Library": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "url": "String",
        "identifier": "List<BackboneElement:Identifier>",
        "version": "String",
        "name": "String",
        "title": "String",
        "subtitle": "String",
        "status": "String",
        "experimental": "Boolean",
        "type": "Concept",
        "subject": "Choice:String",
        "date": "DateTime",
        "publisher": "String",
        "contact": "List<BackboneElement:ContactDetail>",
        "description": "String",
        "useContext": "List<BackboneElement:UsageContext>",
        "jurisdiction": "List<Concept>",
        "purpose": "String",
        "usage": "String",
        "copyright": "String",
        "approvalDate": "Date",
        "lastReviewDate": "Date",
        "effectivePeriod": "Interval<DateTime>",
        "topic": "List<Concept>",
        "author": "List<BackboneElement:ContactDetail>",
        "editor": "List<BackboneElement:ContactDetail>",
        "reviewer": "List<BackboneElement:ContactDetail>",
        "endorser": "List<BackboneElement:ContactDetail>",
        "relatedArtifact": "List<BackboneElement:RelatedArtifact>",
        "parameter": "List<BackboneElement:ParameterDefinition>",
        "dataRequirement": "List<BackboneElement:DataRequirement>",
        "content": "List<BackboneElement:Attachment>",
    },
    "Linkage": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "active": "Boolean",
        "author": "BackboneElement:Reference",
        "item": "List<BackboneElement:Linkage_Item>",
    },
    "List": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "status": "String",
        "mode": "String",
        "title": "String",
        "code": "Concept",
        "subject": "BackboneElement:Reference",
        "encounter": "BackboneElement:Reference",
        "date": "DateTime",
        "source": "BackboneElement:Reference",
        "orderedBy": "Concept",
        "note": "List<BackboneElement:Annotation>",
        "entry": "List<BackboneElement:List_Entry>",
        "emptyReason": "Concept",
    },
    "Location": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "status": "String",
        "operationalStatus": "Code",
        "name": "String",
        "alias": "List<String>",
        "description": "String",
        "mode": "String",
        "type": "List<Concept>",
        "telecom": "List<BackboneElement:ContactPoint>",
        "address": "BackboneElement:Address",
        "physicalType": "Concept",
        "position": "BackboneElement:Location_Position",
        "managingOrganization": "BackboneElement:Reference",
        "partOf": "BackboneElement:Reference",
        "hoursOfOperation": "List<BackboneElement:Location_HoursOfOperation>",
        "availabilityExceptions": "String",
        "endpoint": "List<BackboneElement:Reference>",
    },
    "Measure": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "url": "String",
        "identifier": "List<BackboneElement:Identifier>",
        "version": "String",
        "name": "String",
        "title": "String",
        "subtitle": "String",
        "status": "String",
        "experimental": "Boolean",
        "subject": "Choice:String",
        "date": "DateTime",
        "publisher": "String",
        "contact": "List<BackboneElement:ContactDetail>",
        "description": "String",
        "useContext": "List<BackboneElement:UsageContext>",
        "jurisdiction": "List<Concept>",
        "purpose": "String",
        "usage": "String",
        "copyright": "String",
        "approvalDate": "Date",
        "lastReviewDate": "Date",
        "effectivePeriod": "Interval<DateTime>",
        "topic": "List<Concept>",
        "author": "List<BackboneElement:ContactDetail>",
        "editor": "List<BackboneElement:ContactDetail>",
        "reviewer": "List<BackboneElement:ContactDetail>",
        "endorser": "List<BackboneElement:ContactDetail>",
        "relatedArtifact": "List<BackboneElement:RelatedArtifact>",
        "library": "List<String>",
        "disclaimer": "String",
        "scoring": "Concept",
        "compositeScoring": "Concept",
        "type": "List<Concept>",
        "riskAdjustment": "String",
        "rateAggregation": "String",
        "rationale": "String",
        "clinicalRecommendationStatement": "String",
        "improvementNotation": "Concept",
        "definition": "List<String>",
        "guidance": "String",
        "group": "List<BackboneElement:Measure_Group>",
        "supplementalData": "List<BackboneElement:Measure_SupplementalData>",
    },
    "MeasureReport": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "status": "String",
        "type": "String",
        "measure": "String",
        "subject": "BackboneElement:Reference",
        "date": "DateTime",
        "reporter": "BackboneElement:Reference",
        "period": "Interval<DateTime>",
        "improvementNotation": "Concept",
        "group": "List<BackboneElement:MeasureReport_Group>",
        "evaluatedResource": "List<BackboneElement:Reference>",
    },
    "Media": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "basedOn": "List<BackboneElement:Reference>",
        "partOf": "List<BackboneElement:Reference>",
        "status": "Code",
        "type": "Concept",
        "modality": "Concept",
        "view": "Concept",
        "subject": "BackboneElement:Reference",
        "encounter": "BackboneElement:Reference",
        "created": "Choice:Interval<DateTime>",
        "issued": "String",
        "operator": "BackboneElement:Reference",
        "reasonCode": "List<Concept>",
        "bodySite": "Concept",
        "deviceName": "String",
        "device": "BackboneElement:Reference",
        "height": "String",
        "width": "String",
        "frames": "String",
        "duration": "Decimal",
        "content": "BackboneElement:Attachment",
        "note": "List<BackboneElement:Annotation>",
    },
    "Medication": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "code": "Concept",
        "status": "Code",
        "manufacturer": "BackboneElement:Reference",
        "form": "Concept",
        "amount": "BackboneElement:Ratio",
        "ingredient": "List<BackboneElement:Medication_Ingredient>",
        "batch": "BackboneElement:Medication_Batch",
    },
    "MedicationAdministration": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "instantiates": "List<String>",
        "partOf": "List<BackboneElement:Reference>",
        "status": "Code",
        "statusReason": "List<Concept>",
        "category": "Concept",
        "medication": "Choice:String",
        "subject": "BackboneElement:Reference",
        "context": "BackboneElement:Reference",
        "supportingInformation": "List<BackboneElement:Reference>",
        "effective": "Choice:Interval<DateTime>",
        "performer": "List<BackboneElement:MedicationAdministration_Performer>",
        "reasonCode": "List<Concept>",
        "reasonReference": "List<BackboneElement:Reference>",
        "request": "BackboneElement:Reference",
        "device": "List<BackboneElement:Reference>",
        "note": "List<BackboneElement:Annotation>",
        "dosage": "BackboneElement:MedicationAdministration_Dosage",
        "eventHistory": "List<BackboneElement:Reference>",
    },
    "MedicationDispense": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "partOf": "List<BackboneElement:Reference>",
        "status": "Code",
        "statusReason": "Choice:String",
        "category": "Concept",
        "medication": "Choice:String",
        "subject": "BackboneElement:Reference",
        "context": "BackboneElement:Reference",
        "supportingInformation": "List<BackboneElement:Reference>",
        "performer": "List<BackboneElement:MedicationDispense_Performer>",
        "location": "BackboneElement:Reference",
        "authorizingPrescription": "List<BackboneElement:Reference>",
        "type": "Concept",
        "quantity": "Decimal",
        "daysSupply": "Decimal",
        "whenPrepared": "DateTime",
        "whenHandedOver": "DateTime",
        "destination": "BackboneElement:Reference",
        "receiver": "List<BackboneElement:Reference>",
        "note": "List<BackboneElement:Annotation>",
        "dosageInstruction": "List<BackboneElement:Dosage>",
        "substitution": "BackboneElement:MedicationDispense_Substitution",
        "detectedIssue": "List<BackboneElement:Reference>",
        "eventHistory": "List<BackboneElement:Reference>",
    },
    "MedicationKnowledge": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Concept",
        "status": "Code",
        "manufacturer": "BackboneElement:Reference",
        "doseForm": "Concept",
        "amount": "Decimal",
        "synonym": "List<String>",
        "relatedMedicationKnowledge": "List<BackboneElement:MedicationKnowledge_RelatedMedicationKnowledge>",
        "associatedMedication": "List<BackboneElement:Reference>",
        "productType": "List<Concept>",
        "monograph": "List<BackboneElement:MedicationKnowledge_Monograph>",
        "ingredient": "List<BackboneElement:MedicationKnowledge_Ingredient>",
        "preparationInstruction": "String",
        "intendedRoute": "List<Concept>",
        "cost": "List<BackboneElement:MedicationKnowledge_Cost>",
        "monitoringProgram": "List<BackboneElement:MedicationKnowledge_MonitoringProgram>",
        "administrationGuidelines": "List<BackboneElement:MedicationKnowledge_AdministrationGuidelines>",
        "medicineClassification": "List<BackboneElement:MedicationKnowledge_MedicineClassification>",
        "packaging": "BackboneElement:MedicationKnowledge_Packaging",
        "drugCharacteristic": "List<BackboneElement:MedicationKnowledge_DrugCharacteristic>",
        "contraindication": "List<BackboneElement:Reference>",
        "regulatory": "List<BackboneElement:MedicationKnowledge_Regulatory>",
        "kinetics": "List<BackboneElement:MedicationKnowledge_Kinetics>",
    },
    "MedicationRequest": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "status": "Code",
        "statusReason": "Concept",
        "intent": "Code",
        "category": "List<Concept>",
        "priority": "Code",
        "doNotPerform": "Boolean",
        "reported": "Choice:String",
        "medication": "Choice:String",
        "subject": "BackboneElement:Reference",
        "encounter": "BackboneElement:Reference",
        "supportingInformation": "List<BackboneElement:Reference>",
        "authoredOn": "DateTime",
        "requester": "BackboneElement:Reference",
        "performer": "BackboneElement:Reference",
        "performerType": "Concept",
        "recorder": "BackboneElement:Reference",
        "reasonCode": "List<Concept>",
        "reasonReference": "List<BackboneElement:Reference>",
        "instantiatesCanonical": "List<String>",
        "instantiatesUri": "List<String>",
        "basedOn": "List<BackboneElement:Reference>",
        "groupIdentifier": "BackboneElement:Identifier",
        "courseOfTherapyType": "Concept",
        "insurance": "List<BackboneElement:Reference>",
        "note": "List<BackboneElement:Annotation>",
        "dosageInstruction": "List<BackboneElement:Dosage>",
        "dispenseRequest": "BackboneElement:MedicationRequest_DispenseRequest",
        "substitution": "BackboneElement:MedicationRequest_Substitution",
        "priorPrescription": "BackboneElement:Reference",
        "detectedIssue": "List<BackboneElement:Reference>",
        "eventHistory": "List<BackboneElement:Reference>",
    },
    "MedicationStatement": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "basedOn": "List<BackboneElement:Reference>",
        "partOf": "List<BackboneElement:Reference>",
        "status": "Code",
        "statusReason": "List<Concept>",
        "category": "Concept",
        "medication": "Choice:String",
        "subject": "BackboneElement:Reference",
        "context": "BackboneElement:Reference",
        "effective": "Choice:Interval<DateTime>",
        "dateAsserted": "DateTime",
        "informationSource": "BackboneElement:Reference",
        "derivedFrom": "List<BackboneElement:Reference>",
        "reasonCode": "List<Concept>",
        "reasonReference": "List<BackboneElement:Reference>",
        "note": "List<BackboneElement:Annotation>",
        "dosage": "List<BackboneElement:Dosage>",
    },
    "MedicinalProduct": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "type": "Concept",
        "domain": "Code",
        "combinedPharmaceuticalDoseForm": "Concept",
        "legalStatusOfSupply": "Concept",
        "additionalMonitoringIndicator": "Concept",
        "specialMeasures": "List<String>",
        "paediatricUseIndicator": "Concept",
        "productClassification": "List<Concept>",
        "marketingStatus": "List<BackboneElement:MarketingStatus>",
        "pharmaceuticalProduct": "List<BackboneElement:Reference>",
        "packagedMedicinalProduct": "List<BackboneElement:Reference>",
        "attachedDocument": "List<BackboneElement:Reference>",
        "masterFile": "List<BackboneElement:Reference>",
        "contact": "List<BackboneElement:Reference>",
        "clinicalTrial": "List<BackboneElement:Reference>",
        "name": "List<BackboneElement:MedicinalProduct_Name>",
        "crossReference": "List<BackboneElement:Identifier>",
        "manufacturingBusinessOperation": "List<BackboneElement:MedicinalProduct_ManufacturingBusinessOperation>",
        "specialDesignation": "List<BackboneElement:MedicinalProduct_SpecialDesignation>",
    },
    "MedicinalProductAuthorization": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "subject": "BackboneElement:Reference",
        "country": "List<Concept>",
        "jurisdiction": "List<Concept>",
        "status": "Concept",
        "statusDate": "DateTime",
        "restoreDate": "DateTime",
        "validityPeriod": "Interval<DateTime>",
        "dataExclusivityPeriod": "Interval<DateTime>",
        "dateOfFirstAuthorization": "DateTime",
        "internationalBirthDate": "DateTime",
        "legalBasis": "Concept",
        "jurisdictionalAuthorization": "List<BackboneElement:MedicinalProductAuthorization_JurisdictionalAuthorization>",
        "holder": "BackboneElement:Reference",
        "regulator": "BackboneElement:Reference",
        "procedure": "BackboneElement:MedicinalProductAuthorization_Procedure",
    },
    "MedicinalProductContraindication": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "subject": "List<BackboneElement:Reference>",
        "disease": "Concept",
        "diseaseStatus": "Concept",
        "comorbidity": "List<Concept>",
        "therapeuticIndication": "List<BackboneElement:Reference>",
        "otherTherapy": "List<BackboneElement:MedicinalProductContraindication_OtherTherapy>",
        "population": "List<BackboneElement:Population>",
    },
    "MedicinalProductIndication": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "subject": "List<BackboneElement:Reference>",
        "diseaseSymptomProcedure": "Concept",
        "diseaseStatus": "Concept",
        "comorbidity": "List<Concept>",
        "intendedEffect": "Concept",
        "duration": "Decimal",
        "otherTherapy": "List<BackboneElement:MedicinalProductIndication_OtherTherapy>",
        "undesirableEffect": "List<BackboneElement:Reference>",
        "population": "List<BackboneElement:Population>",
    },
    "MedicinalProductIngredient": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "BackboneElement:Identifier",
        "role": "Concept",
        "allergenicIndicator": "Boolean",
        "manufacturer": "List<BackboneElement:Reference>",
        "specifiedSubstance": "List<BackboneElement:MedicinalProductIngredient_SpecifiedSubstance>",
        "substance": "BackboneElement:MedicinalProductIngredient_Substance",
    },
    "MedicinalProductInteraction": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "subject": "List<BackboneElement:Reference>",
        "description": "String",
        "interactant": "List<BackboneElement:MedicinalProductInteraction_Interactant>",
        "type": "Concept",
        "effect": "Concept",
        "incidence": "Concept",
        "management": "Concept",
    },
    "MedicinalProductManufactured": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "manufacturedDoseForm": "Concept",
        "unitOfPresentation": "Concept",
        "quantity": "Decimal",
        "manufacturer": "List<BackboneElement:Reference>",
        "ingredient": "List<BackboneElement:Reference>",
        "physicalCharacteristics": "BackboneElement:ProdCharacteristic",
        "otherCharacteristics": "List<Concept>",
    },
    "MedicinalProductPackaged": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "subject": "List<BackboneElement:Reference>",
        "description": "String",
        "legalStatusOfSupply": "Concept",
        "marketingStatus": "List<BackboneElement:MarketingStatus>",
        "ma": "Choice:String",
        "batchIdentifier": "List<BackboneElement:MedicinalProductPackaged_BatchIdentifier>",
        "packageItem": "List<BackboneElement:MedicinalProductPackaged_PackageItem>",
    },
    "MedicinalProductPharmaceutical": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "administrableDoseForm": "Concept",
        "unitOfPresentation": "Concept",
        "ingredient": "List<BackboneElement:Reference>",
        "device": "List<BackboneElement:Reference>",
        "characteristics": "List<BackboneElement:MedicinalProductPharmaceutical_Characteristics>",
        "routeOfAdministration": "List<BackboneElement:MedicinalProductPharmaceutical_RouteOfAdministration>",
    },
    "MedicinalProductUndesirableEffect": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "subject": "List<BackboneElement:Reference>",
        "symptomConditionEffect": "Concept",
        "classification": "Concept",
        "frequencyOfOccurrence": "Concept",
        "population": "List<BackboneElement:Population>",
    },
    "MessageDefinition": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "url": "String",
        "identifier": "List<BackboneElement:Identifier>",
        "version": "String",
        "name": "String",
        "title": "String",
        "replaces": "List<String>",
        "status": "String",
        "experimental": "Boolean",
        "date": "DateTime",
        "publisher": "String",
        "contact": "List<BackboneElement:ContactDetail>",
        "description": "String",
        "useContext": "List<BackboneElement:UsageContext>",
        "jurisdiction": "List<Concept>",
        "purpose": "String",
        "copyright": "String",
        "base": "String",
        "parent": "List<String>",
        "event": "Choice:String",
        "category": "String",
        "focus": "List<BackboneElement:MessageDefinition_Focus>",
        "responseRequired": "String",
        "allowedResponse": "List<BackboneElement:MessageDefinition_AllowedResponse>",
        "graph": "List<String>",
    },
    "MessageHeader": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "event": "Choice:String",
        "destination": "List<BackboneElement:MessageHeader_Destination>",
        "sender": "BackboneElement:Reference",
        "enterer": "BackboneElement:Reference",
        "author": "BackboneElement:Reference",
        "source": "BackboneElement:MessageHeader_Source",
        "responsible": "BackboneElement:Reference",
        "reason": "Concept",
        "response": "BackboneElement:MessageHeader_Response",
        "focus": "List<BackboneElement:Reference>",
        "definition": "String",
    },
    "MolecularSequence": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "type": "String",
        "coordinateSystem": "Integer",
        "patient": "BackboneElement:Reference",
        "specimen": "BackboneElement:Reference",
        "device": "BackboneElement:Reference",
        "performer": "BackboneElement:Reference",
        "quantity": "Decimal",
        "referenceSeq": "BackboneElement:MolecularSequence_ReferenceSeq",
        "variant": "List<BackboneElement:MolecularSequence_Variant>",
        "observedSeq": "String",
        "quality": "List<BackboneElement:MolecularSequence_Quality>",
        "readCoverage": "Integer",
        "repository": "List<BackboneElement:MolecularSequence_Repository>",
        "pointer": "List<BackboneElement:Reference>",
        "structureVariant": "List<BackboneElement:MolecularSequence_StructureVariant>",
    },
    "NamingSystem": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "name": "String",
        "status": "String",
        "kind": "String",
        "date": "DateTime",
        "publisher": "String",
        "contact": "List<BackboneElement:ContactDetail>",
        "responsible": "String",
        "type": "Concept",
        "description": "String",
        "useContext": "List<BackboneElement:UsageContext>",
        "jurisdiction": "List<Concept>",
        "usage": "String",
        "uniqueId": "List<BackboneElement:NamingSystem_UniqueId>",
    },
    "NutritionOrder": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "instantiatesCanonical": "List<String>",
        "instantiatesUri": "List<String>",
        "instantiates": "List<String>",
        "status": "Code",
        "intent": "Code",
        "patient": "BackboneElement:Reference",
        "encounter": "BackboneElement:Reference",
        "dateTime": "DateTime",
        "orderer": "BackboneElement:Reference",
        "allergyIntolerance": "List<BackboneElement:Reference>",
        "foodPreferenceModifier": "List<Concept>",
        "excludeFoodModifier": "List<Concept>",
        "oralDiet": "BackboneElement:NutritionOrder_OralDiet",
        "supplement": "List<BackboneElement:NutritionOrder_Supplement>",
        "enteralFormula": "BackboneElement:NutritionOrder_EnteralFormula",
        "note": "List<BackboneElement:Annotation>",
    },
    "Observation": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "basedOn": "List<BackboneElement:Reference>",
        "partOf": "List<BackboneElement:Reference>",
        "status": "String",
        "category": "List<Concept>",
        "code": "Concept",
        "subject": "BackboneElement:Reference",
        "focus": "List<BackboneElement:Reference>",
        "encounter": "BackboneElement:Reference",
        "effective": "Choice:Interval<DateTime>",
        "issued": "String",
        "performer": "List<BackboneElement:Reference>",
        "value": "Choice:Interval<DateTime>",
        "dataAbsentReason": "Concept",
        "interpretation": "List<Concept>",
        "note": "List<BackboneElement:Annotation>",
        "bodySite": "Concept",
        "method": "Concept",
        "specimen": "BackboneElement:Reference",
        "device": "BackboneElement:Reference",
        "referenceRange": "List<BackboneElement:Observation_ReferenceRange>",
        "hasMember": "List<BackboneElement:Reference>",
        "derivedFrom": "List<BackboneElement:Reference>",
        "component": "List<BackboneElement:Observation_Component>",
    },
    "ObservationDefinition": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "category": "List<Concept>",
        "code": "Concept",
        "identifier": "List<BackboneElement:Identifier>",
        "permittedDataType": "List<String>",
        "multipleResultsAllowed": "Boolean",
        "method": "Concept",
        "preferredReportName": "String",
        "quantitativeDetails": "BackboneElement:ObservationDefinition_QuantitativeDetails",
        "qualifiedInterval": "List<BackboneElement:ObservationDefinition_QualifiedInterval>",
        "validCodedValueSet": "BackboneElement:Reference",
        "normalCodedValueSet": "BackboneElement:Reference",
        "abnormalCodedValueSet": "BackboneElement:Reference",
        "criticalCodedValueSet": "BackboneElement:Reference",
    },
    "OperationDefinition": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "url": "String",
        "version": "String",
        "name": "String",
        "title": "String",
        "status": "String",
        "kind": "String",
        "experimental": "Boolean",
        "date": "DateTime",
        "publisher": "String",
        "contact": "List<BackboneElement:ContactDetail>",
        "description": "String",
        "useContext": "List<BackboneElement:UsageContext>",
        "jurisdiction": "List<Concept>",
        "purpose": "String",
        "affectsState": "Boolean",
        "code": "Code",
        "comment": "String",
        "base": "String",
        "resource": "List<Code>",
        "system": "Boolean",
        "type": "Boolean",
        "instance": "Boolean",
        "inputProfile": "String",
        "outputProfile": "String",
        "parameter": "List<BackboneElement:OperationDefinition_Parameter>",
        "overload": "List<BackboneElement:OperationDefinition_Overload>",
    },
    "OperationOutcome": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "issue": "List<BackboneElement:OperationOutcome_Issue>",
    },
    "Organization": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "active": "Boolean",
        "type": "List<Concept>",
        "name": "String",
        "alias": "List<String>",
        "telecom": "List<BackboneElement:ContactPoint>",
        "address": "List<BackboneElement:Address>",
        "partOf": "BackboneElement:Reference",
        "contact": "List<BackboneElement:Organization_Contact>",
        "endpoint": "List<BackboneElement:Reference>",
    },
    "OrganizationAffiliation": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "active": "Boolean",
        "period": "Interval<DateTime>",
        "organization": "BackboneElement:Reference",
        "participatingOrganization": "BackboneElement:Reference",
        "network": "List<BackboneElement:Reference>",
        "code": "List<Concept>",
        "specialty": "List<Concept>",
        "location": "List<BackboneElement:Reference>",
        "healthcareService": "List<BackboneElement:Reference>",
        "telecom": "List<BackboneElement:ContactPoint>",
        "endpoint": "List<BackboneElement:Reference>",
    },
    "Parameters": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "parameter": "List<BackboneElement:Parameters_Parameter>",
    },
    "Patient": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "active": "Boolean",
        "name": "List<BackboneElement:HumanName>",
        "telecom": "List<BackboneElement:ContactPoint>",
        "gender": "String",
        "birthDate": "Date",
        "deceased": "Choice:String",
        "address": "List<BackboneElement:Address>",
        "maritalStatus": "Concept",
        "multipleBirth": "Choice:String",
        "photo": "List<BackboneElement:Attachment>",
        "contact": "List<BackboneElement:Patient_Contact>",
        "communication": "List<BackboneElement:Patient_Communication>",
        "generalPractitioner": "List<BackboneElement:Reference>",
        "managingOrganization": "BackboneElement:Reference",
        "link": "List<BackboneElement:Patient_Link>",
    },
    "PaymentNotice": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "status": "Code",
        "request": "BackboneElement:Reference",
        "response": "BackboneElement:Reference",
        "created": "DateTime",
        "provider": "BackboneElement:Reference",
        "payment": "BackboneElement:Reference",
        "paymentDate": "Date",
        "payee": "BackboneElement:Reference",
        "recipient": "BackboneElement:Reference",
        "amount": "BackboneElement:Money",
        "paymentStatus": "Concept",
    },
    "PaymentReconciliation": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "status": "Code",
        "period": "Interval<DateTime>",
        "created": "DateTime",
        "paymentIssuer": "BackboneElement:Reference",
        "request": "BackboneElement:Reference",
        "requestor": "BackboneElement:Reference",
        "outcome": "String",
        "disposition": "String",
        "paymentDate": "Date",
        "paymentAmount": "BackboneElement:Money",
        "paymentIdentifier": "BackboneElement:Identifier",
        "detail": "List<BackboneElement:PaymentReconciliation_Detail>",
        "formCode": "Concept",
        "processNote": "List<BackboneElement:PaymentReconciliation_ProcessNote>",
    },
    "Person": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "name": "List<BackboneElement:HumanName>",
        "telecom": "List<BackboneElement:ContactPoint>",
        "gender": "String",
        "birthDate": "Date",
        "address": "List<BackboneElement:Address>",
        "photo": "BackboneElement:Attachment",
        "managingOrganization": "BackboneElement:Reference",
        "active": "Boolean",
        "link": "List<BackboneElement:Person_Link>",
    },
    "PlanDefinition": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "url": "String",
        "identifier": "List<BackboneElement:Identifier>",
        "version": "String",
        "name": "String",
        "title": "String",
        "subtitle": "String",
        "type": "Concept",
        "status": "String",
        "experimental": "Boolean",
        "subject": "Choice:String",
        "date": "DateTime",
        "publisher": "String",
        "contact": "List<BackboneElement:ContactDetail>",
        "description": "String",
        "useContext": "List<BackboneElement:UsageContext>",
        "jurisdiction": "List<Concept>",
        "purpose": "String",
        "usage": "String",
        "copyright": "String",
        "approvalDate": "Date",
        "lastReviewDate": "Date",
        "effectivePeriod": "Interval<DateTime>",
        "topic": "List<Concept>",
        "author": "List<BackboneElement:ContactDetail>",
        "editor": "List<BackboneElement:ContactDetail>",
        "reviewer": "List<BackboneElement:ContactDetail>",
        "endorser": "List<BackboneElement:ContactDetail>",
        "relatedArtifact": "List<BackboneElement:RelatedArtifact>",
        "library": "List<String>",
        "goal": "List<BackboneElement:PlanDefinition_Goal>",
        "action": "List<BackboneElement:PlanDefinition_Action>",
    },
    "Practitioner": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "active": "Boolean",
        "name": "List<BackboneElement:HumanName>",
        "telecom": "List<BackboneElement:ContactPoint>",
        "address": "List<BackboneElement:Address>",
        "gender": "String",
        "birthDate": "Date",
        "photo": "List<BackboneElement:Attachment>",
        "qualification": "List<BackboneElement:Practitioner_Qualification>",
        "communication": "List<Concept>",
    },
    "PractitionerRole": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "active": "Boolean",
        "period": "Interval<DateTime>",
        "practitioner": "BackboneElement:Reference",
        "organization": "BackboneElement:Reference",
        "code": "List<Concept>",
        "specialty": "List<Concept>",
        "location": "List<BackboneElement:Reference>",
        "healthcareService": "List<BackboneElement:Reference>",
        "telecom": "List<BackboneElement:ContactPoint>",
        "availableTime": "List<BackboneElement:PractitionerRole_AvailableTime>",
        "notAvailable": "List<BackboneElement:PractitionerRole_NotAvailable>",
        "availabilityExceptions": "String",
        "endpoint": "List<BackboneElement:Reference>",
    },
    "Procedure": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "instantiatesCanonical": "List<String>",
        "instantiatesUri": "List<String>",
        "basedOn": "List<BackboneElement:Reference>",
        "partOf": "List<BackboneElement:Reference>",
        "status": "Code",
        "statusReason": "Concept",
        "category": "Concept",
        "code": "Concept",
        "subject": "BackboneElement:Reference",
        "encounter": "BackboneElement:Reference",
        "performed": "Choice:Interval<DateTime>",
        "recorder": "BackboneElement:Reference",
        "asserter": "BackboneElement:Reference",
        "performer": "List<BackboneElement:Procedure_Performer>",
        "location": "BackboneElement:Reference",
        "reasonCode": "List<Concept>",
        "reasonReference": "List<BackboneElement:Reference>",
        "bodySite": "List<Concept>",
        "outcome": "Concept",
        "report": "List<BackboneElement:Reference>",
        "complication": "List<Concept>",
        "complicationDetail": "List<BackboneElement:Reference>",
        "followUp": "List<Concept>",
        "note": "List<BackboneElement:Annotation>",
        "focalDevice": "List<BackboneElement:Procedure_FocalDevice>",
        "usedReference": "List<BackboneElement:Reference>",
        "usedCode": "List<Concept>",
    },
    "Provenance": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "target": "List<BackboneElement:Reference>",
        "occurred": "Choice:Interval<DateTime>",
        "recorded": "String",
        "policy": "List<String>",
        "location": "BackboneElement:Reference",
        "reason": "List<Concept>",
        "activity": "Concept",
        "agent": "List<BackboneElement:Provenance_Agent>",
        "entity": "List<BackboneElement:Provenance_Entity>",
        "signature": "List<BackboneElement:Signature>",
    },
    "Questionnaire": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "url": "String",
        "identifier": "List<BackboneElement:Identifier>",
        "version": "String",
        "name": "String",
        "title": "String",
        "derivedFrom": "List<String>",
        "status": "String",
        "experimental": "Boolean",
        "subjectType": "List<Code>",
        "date": "DateTime",
        "publisher": "String",
        "contact": "List<BackboneElement:ContactDetail>",
        "description": "String",
        "useContext": "List<BackboneElement:UsageContext>",
        "jurisdiction": "List<Concept>",
        "purpose": "String",
        "copyright": "String",
        "approvalDate": "Date",
        "lastReviewDate": "Date",
        "effectivePeriod": "Interval<DateTime>",
        "code": "List<Code>",
        "item": "List<BackboneElement:Questionnaire_Item>",
    },
    "QuestionnaireResponse": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "BackboneElement:Identifier",
        "basedOn": "List<BackboneElement:Reference>",
        "partOf": "List<BackboneElement:Reference>",
        "questionnaire": "String",
        "status": "String",
        "subject": "BackboneElement:Reference",
        "encounter": "BackboneElement:Reference",
        "authored": "DateTime",
        "author": "BackboneElement:Reference",
        "source": "BackboneElement:Reference",
        "item": "List<BackboneElement:QuestionnaireResponse_Item>",
    },
    "RelatedPerson": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "active": "Boolean",
        "patient": "BackboneElement:Reference",
        "relationship": "List<Concept>",
        "name": "List<BackboneElement:HumanName>",
        "telecom": "List<BackboneElement:ContactPoint>",
        "gender": "String",
        "birthDate": "Date",
        "address": "List<BackboneElement:Address>",
        "photo": "List<BackboneElement:Attachment>",
        "period": "Interval<DateTime>",
        "communication": "List<BackboneElement:RelatedPerson_Communication>",
    },
    "RequestGroup": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "instantiatesCanonical": "List<String>",
        "instantiatesUri": "List<String>",
        "basedOn": "List<BackboneElement:Reference>",
        "replaces": "List<BackboneElement:Reference>",
        "groupIdentifier": "BackboneElement:Identifier",
        "status": "Code",
        "intent": "Code",
        "priority": "Code",
        "code": "Concept",
        "subject": "BackboneElement:Reference",
        "encounter": "BackboneElement:Reference",
        "authoredOn": "DateTime",
        "author": "BackboneElement:Reference",
        "reasonCode": "List<Concept>",
        "reasonReference": "List<BackboneElement:Reference>",
        "note": "List<BackboneElement:Annotation>",
        "action": "List<BackboneElement:RequestGroup_Action>",
    },
    "ResearchDefinition": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "url": "String",
        "identifier": "List<BackboneElement:Identifier>",
        "version": "String",
        "name": "String",
        "title": "String",
        "shortTitle": "String",
        "subtitle": "String",
        "status": "String",
        "experimental": "Boolean",
        "subject": "Choice:String",
        "date": "DateTime",
        "publisher": "String",
        "contact": "List<BackboneElement:ContactDetail>",
        "description": "String",
        "comment": "List<String>",
        "useContext": "List<BackboneElement:UsageContext>",
        "jurisdiction": "List<Concept>",
        "purpose": "String",
        "usage": "String",
        "copyright": "String",
        "approvalDate": "Date",
        "lastReviewDate": "Date",
        "effectivePeriod": "Interval<DateTime>",
        "topic": "List<Concept>",
        "author": "List<BackboneElement:ContactDetail>",
        "editor": "List<BackboneElement:ContactDetail>",
        "reviewer": "List<BackboneElement:ContactDetail>",
        "endorser": "List<BackboneElement:ContactDetail>",
        "relatedArtifact": "List<BackboneElement:RelatedArtifact>",
        "library": "List<String>",
        "population": "BackboneElement:Reference",
        "exposure": "BackboneElement:Reference",
        "exposureAlternative": "BackboneElement:Reference",
        "outcome": "BackboneElement:Reference",
    },
    "ResearchElementDefinition": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "url": "String",
        "identifier": "List<BackboneElement:Identifier>",
        "version": "String",
        "name": "String",
        "title": "String",
        "shortTitle": "String",
        "subtitle": "String",
        "status": "String",
        "experimental": "Boolean",
        "subject": "Choice:String",
        "date": "DateTime",
        "publisher": "String",
        "contact": "List<BackboneElement:ContactDetail>",
        "description": "String",
        "comment": "List<String>",
        "useContext": "List<BackboneElement:UsageContext>",
        "jurisdiction": "List<Concept>",
        "purpose": "String",
        "usage": "String",
        "copyright": "String",
        "approvalDate": "Date",
        "lastReviewDate": "Date",
        "effectivePeriod": "Interval<DateTime>",
        "topic": "List<Concept>",
        "author": "List<BackboneElement:ContactDetail>",
        "editor": "List<BackboneElement:ContactDetail>",
        "reviewer": "List<BackboneElement:ContactDetail>",
        "endorser": "List<BackboneElement:ContactDetail>",
        "relatedArtifact": "List<BackboneElement:RelatedArtifact>",
        "library": "List<String>",
        "type": "String",
        "variableType": "String",
        "characteristic": "List<BackboneElement:ResearchElementDefinition_Characteristic>",
    },
    "ResearchStudy": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "title": "String",
        "protocol": "List<BackboneElement:Reference>",
        "partOf": "List<BackboneElement:Reference>",
        "status": "String",
        "primaryPurposeType": "Concept",
        "phase": "Concept",
        "category": "List<Concept>",
        "focus": "List<Concept>",
        "condition": "List<Concept>",
        "contact": "List<BackboneElement:ContactDetail>",
        "relatedArtifact": "List<BackboneElement:RelatedArtifact>",
        "keyword": "List<Concept>",
        "location": "List<Concept>",
        "description": "String",
        "enrollment": "List<BackboneElement:Reference>",
        "period": "Interval<DateTime>",
        "sponsor": "BackboneElement:Reference",
        "principalInvestigator": "BackboneElement:Reference",
        "site": "List<BackboneElement:Reference>",
        "reasonStopped": "Concept",
        "note": "List<BackboneElement:Annotation>",
        "arm": "List<BackboneElement:ResearchStudy_Arm>",
        "objective": "List<BackboneElement:ResearchStudy_Objective>",
    },
    "ResearchSubject": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "status": "String",
        "period": "Interval<DateTime>",
        "study": "BackboneElement:Reference",
        "individual": "BackboneElement:Reference",
        "assignedArm": "String",
        "actualArm": "String",
        "consent": "BackboneElement:Reference",
    },
    "RiskAssessment": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "basedOn": "BackboneElement:Reference",
        "parent": "BackboneElement:Reference",
        "status": "Code",
        "method": "Concept",
        "code": "Concept",
        "subject": "BackboneElement:Reference",
        "encounter": "BackboneElement:Reference",
        "occurrence": "Choice:Interval<DateTime>",
        "condition": "BackboneElement:Reference",
        "performer": "BackboneElement:Reference",
        "reasonCode": "List<Concept>",
        "reasonReference": "List<BackboneElement:Reference>",
        "basis": "List<BackboneElement:Reference>",
        "prediction": "List<BackboneElement:RiskAssessment_Prediction>",
        "mitigation": "String",
        "note": "List<BackboneElement:Annotation>",
    },
    "RiskEvidenceSynthesis": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "url": "String",
        "identifier": "List<BackboneElement:Identifier>",
        "version": "String",
        "name": "String",
        "title": "String",
        "status": "String",
        "date": "DateTime",
        "publisher": "String",
        "contact": "List<BackboneElement:ContactDetail>",
        "description": "String",
        "note": "List<BackboneElement:Annotation>",
        "useContext": "List<BackboneElement:UsageContext>",
        "jurisdiction": "List<Concept>",
        "copyright": "String",
        "approvalDate": "Date",
        "lastReviewDate": "Date",
        "effectivePeriod": "Interval<DateTime>",
        "topic": "List<Concept>",
        "author": "List<BackboneElement:ContactDetail>",
        "editor": "List<BackboneElement:ContactDetail>",
        "reviewer": "List<BackboneElement:ContactDetail>",
        "endorser": "List<BackboneElement:ContactDetail>",
        "relatedArtifact": "List<BackboneElement:RelatedArtifact>",
        "synthesisType": "Concept",
        "studyType": "Concept",
        "population": "BackboneElement:Reference",
        "exposure": "BackboneElement:Reference",
        "outcome": "BackboneElement:Reference",
        "sampleSize": "BackboneElement:RiskEvidenceSynthesis_SampleSize",
        "riskEstimate": "BackboneElement:RiskEvidenceSynthesis_RiskEstimate",
        "certainty": "List<BackboneElement:RiskEvidenceSynthesis_Certainty>",
    },
    "Schedule": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "active": "Boolean",
        "serviceCategory": "List<Concept>",
        "serviceType": "List<Concept>",
        "specialty": "List<Concept>",
        "actor": "List<BackboneElement:Reference>",
        "planningHorizon": "Interval<DateTime>",
        "comment": "String",
    },
    "SearchParameter": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "url": "String",
        "version": "String",
        "name": "String",
        "derivedFrom": "String",
        "status": "String",
        "experimental": "Boolean",
        "date": "DateTime",
        "publisher": "String",
        "contact": "List<BackboneElement:ContactDetail>",
        "description": "String",
        "useContext": "List<BackboneElement:UsageContext>",
        "jurisdiction": "List<Concept>",
        "purpose": "String",
        "code": "Code",
        "base": "List<Code>",
        "type": "String",
        "expression": "String",
        "xpath": "String",
        "xpathUsage": "String",
        "target": "List<Code>",
        "multipleOr": "Boolean",
        "multipleAnd": "Boolean",
        "comparator": "List<String>",
        "modifier": "List<String>",
        "chain": "List<String>",
        "component": "List<BackboneElement:SearchParameter_Component>",
    },
    "ServiceRequest": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "instantiatesCanonical": "List<String>",
        "instantiatesUri": "List<String>",
        "basedOn": "List<BackboneElement:Reference>",
        "replaces": "List<BackboneElement:Reference>",
        "requisition": "BackboneElement:Identifier",
        "status": "Code",
        "intent": "Code",
        "category": "List<Concept>",
        "priority": "Code",
        "doNotPerform": "Boolean",
        "code": "Concept",
        "orderDetail": "List<Concept>",
        "quantity": "Choice:String",
        "subject": "BackboneElement:Reference",
        "encounter": "BackboneElement:Reference",
        "occurrence": "Choice:Interval<DateTime>",
        "asNeeded": "Choice:String",
        "authoredOn": "DateTime",
        "requester": "BackboneElement:Reference",
        "performerType": "Concept",
        "performer": "List<BackboneElement:Reference>",
        "locationCode": "List<Concept>",
        "locationReference": "List<BackboneElement:Reference>",
        "reasonCode": "List<Concept>",
        "reasonReference": "List<BackboneElement:Reference>",
        "insurance": "List<BackboneElement:Reference>",
        "supportingInfo": "List<BackboneElement:Reference>",
        "specimen": "List<BackboneElement:Reference>",
        "bodySite": "List<Concept>",
        "note": "List<BackboneElement:Annotation>",
        "patientInstruction": "String",
        "relevantHistory": "List<BackboneElement:Reference>",
    },
    "Slot": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "serviceCategory": "List<Concept>",
        "serviceType": "List<Concept>",
        "specialty": "List<Concept>",
        "appointmentType": "Concept",
        "schedule": "BackboneElement:Reference",
        "status": "String",
        "start": "String",
        "end": "String",
        "overbooked": "Boolean",
        "comment": "String",
    },
    "Specimen": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "accessionIdentifier": "BackboneElement:Identifier",
        "status": "String",
        "type": "Concept",
        "subject": "BackboneElement:Reference",
        "receivedTime": "DateTime",
        "parent": "List<BackboneElement:Reference>",
        "request": "List<BackboneElement:Reference>",
        "collection": "BackboneElement:Specimen_Collection",
        "processing": "List<BackboneElement:Specimen_Processing>",
        "container": "List<BackboneElement:Specimen_Container>",
        "condition": "List<Concept>",
        "note": "List<BackboneElement:Annotation>",
    },
    "SpecimenDefinition": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "BackboneElement:Identifier",
        "typeCollected": "Concept",
        "patientPreparation": "List<Concept>",
        "timeAspect": "String",
        "collection": "List<Concept>",
        "typeTested": "List<BackboneElement:SpecimenDefinition_TypeTested>",
    },
    "StructureDefinition": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "url": "String",
        "identifier": "List<BackboneElement:Identifier>",
        "version": "String",
        "name": "String",
        "title": "String",
        "status": "String",
        "experimental": "Boolean",
        "date": "DateTime",
        "publisher": "String",
        "contact": "List<BackboneElement:ContactDetail>",
        "description": "String",
        "useContext": "List<BackboneElement:UsageContext>",
        "jurisdiction": "List<Concept>",
        "purpose": "String",
        "copyright": "String",
        "keyword": "List<Code>",
        "fhirVersion": "String",
        "mapping": "List<BackboneElement:StructureDefinition_Mapping>",
        "kind": "String",
        "abstract": "Boolean",
        "context": "List<BackboneElement:StructureDefinition_Context>",
        "contextInvariant": "List<String>",
        "type": "String",
        "baseDefinition": "String",
        "derivation": "String",
        "snapshot": "BackboneElement:StructureDefinition_Snapshot",
        "differential": "BackboneElement:StructureDefinition_Differential",
    },
    "StructureMap": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "url": "String",
        "identifier": "List<BackboneElement:Identifier>",
        "version": "String",
        "name": "String",
        "title": "String",
        "status": "String",
        "experimental": "Boolean",
        "date": "DateTime",
        "publisher": "String",
        "contact": "List<BackboneElement:ContactDetail>",
        "description": "String",
        "useContext": "List<BackboneElement:UsageContext>",
        "jurisdiction": "List<Concept>",
        "purpose": "String",
        "copyright": "String",
        "structure": "List<BackboneElement:StructureMap_Structure>",
        "import": "List<String>",
        "group": "List<BackboneElement:StructureMap_Group>",
    },
    "Subscription": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "status": "String",
        "contact": "List<BackboneElement:ContactPoint>",
        "end": "String",
        "reason": "String",
        "criteria": "String",
        "error": "String",
        "channel": "BackboneElement:Subscription_Channel",
    },
    "Substance": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "status": "String",
        "category": "List<Concept>",
        "code": "Concept",
        "description": "String",
        "instance": "List<BackboneElement:Substance_Instance>",
        "ingredient": "List<BackboneElement:Substance_Ingredient>",
    },
    "SubstanceNucleicAcid": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "sequenceType": "Concept",
        "numberOfSubunits": "Integer",
        "areaOfHybridisation": "String",
        "oligoNucleotideType": "Concept",
        "subunit": "List<BackboneElement:SubstanceNucleicAcid_Subunit>",
    },
    "SubstancePolymer": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "": "Choice:String",
    },
    "SubstanceProtein": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "sequenceType": "Concept",
        "numberOfSubunits": "Integer",
        "disulfideLinkage": "List<String>",
        "subunit": "List<BackboneElement:SubstanceProtein_Subunit>",
    },
    "SubstanceReferenceInformation": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "": "Choice:String",
    },
    "SubstanceSourceMaterial": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "sourceMaterialClass": "Concept",
        "sourceMaterialType": "Concept",
        "sourceMaterialState": "Concept",
        "organismId": "BackboneElement:Identifier",
        "organismName": "String",
        "parentSubstanceId": "List<BackboneElement:Identifier>",
        "parentSubstanceName": "List<String>",
        "countryOfOrigin": "List<Concept>",
        "geographicalLocation": "List<String>",
        "developmentStage": "Concept",
        "fractionDescription": "List<BackboneElement:SubstanceSourceMaterial_FractionDescription>",
        "organism": "BackboneElement:SubstanceSourceMaterial_Organism",
        "partDescription": "List<BackboneElement:SubstanceSourceMaterial_PartDescription>",
    },
    "SubstanceSpecification": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "BackboneElement:Identifier",
        "type": "Concept",
        "status": "Concept",
        "domain": "Concept",
        "description": "String",
        "source": "List<BackboneElement:Reference>",
        "comment": "String",
        "moiety": "List<BackboneElement:SubstanceSpecification_Moiety>",
        "property": "List<BackboneElement:SubstanceSpecification_Property>",
        "referenceInformation": "BackboneElement:Reference",
        "structure": "BackboneElement:SubstanceSpecification_Structure",
        "code": "List<BackboneElement:SubstanceSpecification_Code>",
        "name": "List<BackboneElement:SubstanceSpecification_Name>",
        "molecularWeight": "List<BackboneElement:SubstanceSpecification_MolecularWeight>",
        "relationship": "List<BackboneElement:SubstanceSpecification_Relationship>",
        "nucleicAcid": "BackboneElement:Reference",
        "polymer": "BackboneElement:Reference",
        "protein": "BackboneElement:Reference",
        "sourceMaterial": "BackboneElement:Reference",
    },
    "SupplyDelivery": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "basedOn": "List<BackboneElement:Reference>",
        "partOf": "List<BackboneElement:Reference>",
        "status": "String",
        "patient": "BackboneElement:Reference",
        "type": "Concept",
        "suppliedItem": "BackboneElement:SupplyDelivery_SuppliedItem",
        "occurrence": "Choice:Interval<DateTime>",
        "supplier": "BackboneElement:Reference",
        "destination": "BackboneElement:Reference",
        "receiver": "List<BackboneElement:Reference>",
    },
    "SupplyRequest": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "status": "String",
        "category": "Concept",
        "priority": "Code",
        "item": "Choice:String",
        "quantity": "Decimal",
        "parameter": "List<BackboneElement:SupplyRequest_Parameter>",
        "occurrence": "Choice:Interval<DateTime>",
        "authoredOn": "DateTime",
        "requester": "BackboneElement:Reference",
        "supplier": "List<BackboneElement:Reference>",
        "reason": "Choice:String",
        "deliverFrom": "BackboneElement:Reference",
        "deliverTo": "BackboneElement:Reference",
    },
    "Task": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "instantiatesCanonical": "String",
        "instantiatesUri": "String",
        "basedOn": "List<BackboneElement:Reference>",
        "groupIdentifier": "BackboneElement:Identifier",
        "partOf": "List<BackboneElement:Reference>",
        "status": "String",
        "statusReason": "Concept",
        "businessStatus": "Concept",
        "intent": "String",
        "priority": "Code",
        "code": "Concept",
        "description": "String",
        "focus": "BackboneElement:Reference",
        "for": "BackboneElement:Reference",
        "encounter": "BackboneElement:Reference",
        "executionPeriod": "Interval<DateTime>",
        "authoredOn": "DateTime",
        "lastModified": "DateTime",
        "requester": "BackboneElement:Reference",
        "performerType": "List<Concept>",
        "owner": "BackboneElement:Reference",
        "location": "BackboneElement:Reference",
        "reasonCode": "Concept",
        "reasonReference": "BackboneElement:Reference",
        "insurance": "List<BackboneElement:Reference>",
        "note": "List<BackboneElement:Annotation>",
        "relevantHistory": "List<BackboneElement:Reference>",
        "restriction": "BackboneElement:Task_Restriction",
        "input": "List<BackboneElement:Task_Input>",
        "output": "List<BackboneElement:Task_Output>",
    },
    "TerminologyCapabilities": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "url": "String",
        "version": "String",
        "name": "String",
        "title": "String",
        "status": "String",
        "experimental": "Boolean",
        "date": "DateTime",
        "publisher": "String",
        "contact": "List<BackboneElement:ContactDetail>",
        "description": "String",
        "useContext": "List<BackboneElement:UsageContext>",
        "jurisdiction": "List<Concept>",
        "purpose": "String",
        "copyright": "String",
        "kind": "Code",
        "software": "BackboneElement:TerminologyCapabilities_Software",
        "implementation": "BackboneElement:TerminologyCapabilities_Implementation",
        "lockedDate": "Boolean",
        "codeSystem": "List<BackboneElement:TerminologyCapabilities_CodeSystem>",
        "expansion": "BackboneElement:TerminologyCapabilities_Expansion",
        "codeSearch": "String",
        "validateCode": "BackboneElement:TerminologyCapabilities_ValidateCode",
        "translation": "BackboneElement:TerminologyCapabilities_Translation",
        "closure": "BackboneElement:TerminologyCapabilities_Closure",
    },
    "TestReport": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "BackboneElement:Identifier",
        "name": "String",
        "status": "String",
        "testScript": "BackboneElement:Reference",
        "result": "String",
        "score": "Decimal",
        "tester": "String",
        "issued": "DateTime",
        "participant": "List<BackboneElement:TestReport_Participant>",
        "setup": "BackboneElement:TestReport_Setup",
        "test": "List<BackboneElement:TestReport_Test>",
        "teardown": "BackboneElement:TestReport_Teardown",
    },
    "TestScript": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "url": "String",
        "identifier": "BackboneElement:Identifier",
        "version": "String",
        "name": "String",
        "title": "String",
        "status": "String",
        "experimental": "Boolean",
        "date": "DateTime",
        "publisher": "String",
        "contact": "List<BackboneElement:ContactDetail>",
        "description": "String",
        "useContext": "List<BackboneElement:UsageContext>",
        "jurisdiction": "List<Concept>",
        "purpose": "String",
        "copyright": "String",
        "origin": "List<BackboneElement:TestScript_Origin>",
        "destination": "List<BackboneElement:TestScript_Destination>",
        "metadata": "BackboneElement:TestScript_Metadata",
        "fixture": "List<BackboneElement:TestScript_Fixture>",
        "profile": "List<BackboneElement:Reference>",
        "variable": "List<BackboneElement:TestScript_Variable>",
        "setup": "BackboneElement:TestScript_Setup",
        "test": "List<BackboneElement:TestScript_Test>",
        "teardown": "BackboneElement:TestScript_Teardown",
    },
    "ValueSet": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "url": "String",
        "identifier": "List<BackboneElement:Identifier>",
        "version": "String",
        "name": "String",
        "title": "String",
        "status": "String",
        "experimental": "Boolean",
        "date": "DateTime",
        "publisher": "String",
        "contact": "List<BackboneElement:ContactDetail>",
        "description": "String",
        "useContext": "List<BackboneElement:UsageContext>",
        "jurisdiction": "List<Concept>",
        "immutable": "Boolean",
        "purpose": "String",
        "copyright": "String",
        "compose": "BackboneElement:ValueSet_Compose",
        "expansion": "BackboneElement:ValueSet_Expansion",
    },
    "VerificationResult": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "target": "List<BackboneElement:Reference>",
        "targetLocation": "List<String>",
        "need": "Concept",
        "status": "Code",
        "statusDate": "DateTime",
        "validationType": "Concept",
        "validationProcess": "List<Concept>",
        "frequency": "BackboneElement:Timing",
        "lastPerformed": "DateTime",
        "nextScheduled": "Date",
        "failureAction": "Concept",
        "primarySource": "List<BackboneElement:VerificationResult_PrimarySource>",
        "attestation": "BackboneElement:VerificationResult_Attestation",
        "validator": "List<BackboneElement:VerificationResult_Validator>",
    },
    "VisionPrescription": {
        "id": "String",
        "meta": "BackboneElement:Meta",
        "implicitRules": "String",
        "language": "Code",
        "text": "BackboneElement:Narrative",
        "contained": "List<String>",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "status": "Code",
        "created": "DateTime",
        "patient": "BackboneElement:Reference",
        "encounter": "BackboneElement:Reference",
        "dateWritten": "DateTime",
        "prescriber": "BackboneElement:Reference",
        "lensSpecification": "List<BackboneElement:VisionPrescription_LensSpecification>",
    },
    "Account_Coverage": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "coverage": "BackboneElement:Reference",
        "priority": "String",
    },
    "Account_Guarantor": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "party": "BackboneElement:Reference",
        "onHold": "Boolean",
        "period": "Interval<DateTime>",
    },
    "ActivityDefinition_DynamicValue": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "path": "String",
        "expression": "BackboneElement:Expression",
    },
    "ActivityDefinition_Participant": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Code",
        "role": "Concept",
    },
    "Address": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "use": "String",
        "type": "String",
        "text": "String",
        "line": "List<String>",
        "city": "String",
        "district": "String",
        "state": "String",
        "postalCode": "String",
        "country": "String",
        "period": "Interval<DateTime>",
    },
    "AdverseEvent_Causality": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "assessment": "Concept",
        "productRelatedness": "String",
        "author": "BackboneElement:Reference",
        "method": "Concept",
    },
    "AdverseEvent_SuspectEntity": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "instance": "BackboneElement:Reference",
        "causality": "List<BackboneElement:AdverseEvent_Causality>",
    },
    "Age": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "value": "Decimal",
        "comparator": "String",
        "unit": "String",
        "system": "String",
        "code": "Code",
    },
    "AllergyIntolerance_Reaction": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "substance": "Concept",
        "manifestation": "List<Concept>",
        "description": "String",
        "onset": "DateTime",
        "severity": "String",
        "exposureRoute": "Concept",
        "note": "List<BackboneElement:Annotation>",
    },
    "Annotation": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "author": "Choice:String",
        "time": "DateTime",
        "text": "String",
    },
    "Appointment_Participant": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "List<Concept>",
        "actor": "BackboneElement:Reference",
        "required": "String",
        "status": "String",
        "period": "Interval<DateTime>",
    },
    "Attachment": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "contentType": "Code",
        "language": "Code",
        "data": "String",
        "url": "String",
        "size": "String",
        "hash": "String",
        "title": "String",
        "creation": "DateTime",
    },
    "AuditEvent_Agent": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "role": "List<Concept>",
        "who": "BackboneElement:Reference",
        "altId": "String",
        "name": "String",
        "requestor": "Boolean",
        "location": "BackboneElement:Reference",
        "policy": "List<String>",
        "media": "Code",
        "network": "BackboneElement:AuditEvent_Network",
        "purposeOfUse": "List<Concept>",
    },
    "AuditEvent_Detail": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "String",
        "value": "Choice:String",
    },
    "AuditEvent_Entity": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "what": "BackboneElement:Reference",
        "type": "Code",
        "role": "Code",
        "lifecycle": "Code",
        "securityLabel": "List<Code>",
        "name": "String",
        "description": "String",
        "query": "String",
        "detail": "List<BackboneElement:AuditEvent_Detail>",
    },
    "AuditEvent_Network": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "address": "String",
        "type": "String",
    },
    "AuditEvent_Source": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "site": "String",
        "observer": "BackboneElement:Reference",
        "type": "List<Code>",
    },
    "BiologicallyDerivedProduct_Collection": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "collector": "BackboneElement:Reference",
        "source": "BackboneElement:Reference",
        "collected": "Choice:Interval<DateTime>",
    },
    "BiologicallyDerivedProduct_Manipulation": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "description": "String",
        "time": "Choice:Interval<DateTime>",
    },
    "BiologicallyDerivedProduct_Processing": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "description": "String",
        "procedure": "Concept",
        "additive": "BackboneElement:Reference",
        "time": "Choice:Interval<DateTime>",
    },
    "BiologicallyDerivedProduct_Storage": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "description": "String",
        "temperature": "Decimal",
        "scale": "String",
        "duration": "Interval<DateTime>",
    },
    "Bundle_Entry": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "link": "List<BackboneElement:Bundle_Link>",
        "fullUrl": "String",
        "resource": "String",
        "search": "BackboneElement:Bundle_Search",
        "request": "BackboneElement:Bundle_Request",
        "response": "BackboneElement:Bundle_Response",
    },
    "Bundle_Link": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "relation": "String",
        "url": "String",
    },
    "Bundle_Request": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "method": "String",
        "url": "String",
        "ifNoneMatch": "String",
        "ifModifiedSince": "String",
        "ifMatch": "String",
        "ifNoneExist": "String",
    },
    "Bundle_Response": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "status": "String",
        "location": "String",
        "etag": "String",
        "lastModified": "String",
        "outcome": "String",
    },
    "Bundle_Search": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "mode": "String",
        "score": "Decimal",
    },
    "CapabilityStatement_Document": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "mode": "String",
        "documentation": "String",
        "profile": "String",
    },
    "CapabilityStatement_Endpoint": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "protocol": "Code",
        "address": "String",
    },
    "CapabilityStatement_Implementation": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "description": "String",
        "url": "String",
        "custodian": "BackboneElement:Reference",
    },
    "CapabilityStatement_Interaction": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "String",
        "documentation": "String",
    },
    "CapabilityStatement_Interaction1": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "String",
        "documentation": "String",
    },
    "CapabilityStatement_Messaging": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "endpoint": "List<BackboneElement:CapabilityStatement_Endpoint>",
        "reliableCache": "String",
        "documentation": "String",
        "supportedMessage": "List<BackboneElement:CapabilityStatement_SupportedMessage>",
    },
    "CapabilityStatement_Operation": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "name": "String",
        "definition": "String",
        "documentation": "String",
    },
    "CapabilityStatement_Resource": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Code",
        "profile": "String",
        "supportedProfile": "List<String>",
        "documentation": "String",
        "interaction": "List<BackboneElement:CapabilityStatement_Interaction>",
        "versioning": "String",
        "readHistory": "Boolean",
        "updateCreate": "Boolean",
        "conditionalCreate": "Boolean",
        "conditionalRead": "String",
        "conditionalUpdate": "Boolean",
        "conditionalDelete": "String",
        "referencePolicy": "List<String>",
        "searchInclude": "List<String>",
        "searchRevInclude": "List<String>",
        "searchParam": "List<BackboneElement:CapabilityStatement_SearchParam>",
        "operation": "List<BackboneElement:CapabilityStatement_Operation>",
    },
    "CapabilityStatement_Rest": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "mode": "String",
        "documentation": "String",
        "security": "BackboneElement:CapabilityStatement_Security",
        "resource": "List<BackboneElement:CapabilityStatement_Resource>",
        "interaction": "List<BackboneElement:CapabilityStatement_Interaction1>",
        "searchParam": "List<BackboneElement:CapabilityStatement_SearchParam>",
        "operation": "List<BackboneElement:CapabilityStatement_Operation>",
        "compartment": "List<String>",
    },
    "CapabilityStatement_SearchParam": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "name": "String",
        "definition": "String",
        "type": "String",
        "documentation": "String",
    },
    "CapabilityStatement_Security": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "cors": "Boolean",
        "service": "List<Concept>",
        "description": "String",
    },
    "CapabilityStatement_Software": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "name": "String",
        "version": "String",
        "releaseDate": "DateTime",
    },
    "CapabilityStatement_SupportedMessage": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "mode": "String",
        "definition": "String",
    },
    "CarePlan_Activity": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "outcomeCodeableConcept": "List<Concept>",
        "outcomeReference": "List<BackboneElement:Reference>",
        "progress": "List<BackboneElement:Annotation>",
        "reference": "BackboneElement:Reference",
        "detail": "BackboneElement:CarePlan_Detail",
    },
    "CarePlan_Detail": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "kind": "Code",
        "instantiatesCanonical": "List<String>",
        "instantiatesUri": "List<String>",
        "code": "Concept",
        "reasonCode": "List<Concept>",
        "reasonReference": "List<BackboneElement:Reference>",
        "goal": "List<BackboneElement:Reference>",
        "status": "String",
        "statusReason": "Concept",
        "doNotPerform": "Boolean",
        "scheduled": "Choice:Interval<DateTime>",
        "location": "BackboneElement:Reference",
        "performer": "List<BackboneElement:Reference>",
        "product": "Choice:String",
        "dailyAmount": "Decimal",
        "quantity": "Decimal",
        "description": "String",
    },
    "CareTeam_Participant": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "role": "List<Concept>",
        "member": "BackboneElement:Reference",
        "onBehalfOf": "BackboneElement:Reference",
        "period": "Interval<DateTime>",
    },
    "CatalogEntry_RelatedEntry": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "relationtype": "String",
        "item": "BackboneElement:Reference",
    },
    "ChargeItemDefinition_Applicability": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "description": "String",
        "language": "String",
        "expression": "String",
    },
    "ChargeItemDefinition_PriceComponent": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Code",
        "code": "Concept",
        "factor": "Decimal",
        "amount": "BackboneElement:Money",
    },
    "ChargeItemDefinition_PropertyGroup": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "applicability": "List<BackboneElement:ChargeItemDefinition_Applicability>",
        "priceComponent": "List<BackboneElement:ChargeItemDefinition_PriceComponent>",
    },
    "ChargeItem_Performer": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "function": "Concept",
        "actor": "BackboneElement:Reference",
    },
    "ClaimResponse_AddItem": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "itemSequence": "List<String>",
        "detailSequence": "List<String>",
        "subdetailSequence": "List<String>",
        "provider": "List<BackboneElement:Reference>",
        "productOrService": "Concept",
        "modifier": "List<Concept>",
        "programCode": "List<Concept>",
        "serviced": "Choice:Interval<DateTime>",
        "location": "Choice:String",
        "quantity": "Decimal",
        "unitPrice": "BackboneElement:Money",
        "factor": "Decimal",
        "net": "BackboneElement:Money",
        "bodySite": "Concept",
        "subSite": "List<Concept>",
        "noteNumber": "List<String>",
        "adjudication": "List<BackboneElement:ClaimResponse_Adjudication>",
        "detail": "List<BackboneElement:ClaimResponse_Detail1>",
    },
    "ClaimResponse_Adjudication": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "category": "Concept",
        "reason": "Concept",
        "amount": "BackboneElement:Money",
        "value": "Decimal",
    },
    "ClaimResponse_Detail": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "detailSequence": "String",
        "noteNumber": "List<String>",
        "adjudication": "List<BackboneElement:ClaimResponse_Adjudication>",
        "subDetail": "List<BackboneElement:ClaimResponse_SubDetail>",
    },
    "ClaimResponse_Detail1": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "productOrService": "Concept",
        "modifier": "List<Concept>",
        "quantity": "Decimal",
        "unitPrice": "BackboneElement:Money",
        "factor": "Decimal",
        "net": "BackboneElement:Money",
        "noteNumber": "List<String>",
        "adjudication": "List<BackboneElement:ClaimResponse_Adjudication>",
        "subDetail": "List<BackboneElement:ClaimResponse_SubDetail1>",
    },
    "ClaimResponse_Error": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "itemSequence": "String",
        "detailSequence": "String",
        "subDetailSequence": "String",
        "code": "Concept",
    },
    "ClaimResponse_Insurance": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "sequence": "String",
        "focal": "Boolean",
        "coverage": "BackboneElement:Reference",
        "businessArrangement": "String",
        "claimResponse": "BackboneElement:Reference",
    },
    "ClaimResponse_Item": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "itemSequence": "String",
        "noteNumber": "List<String>",
        "adjudication": "List<BackboneElement:ClaimResponse_Adjudication>",
        "detail": "List<BackboneElement:ClaimResponse_Detail>",
    },
    "ClaimResponse_Payment": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "adjustment": "BackboneElement:Money",
        "adjustmentReason": "Concept",
        "date": "Date",
        "amount": "BackboneElement:Money",
        "identifier": "BackboneElement:Identifier",
    },
    "ClaimResponse_ProcessNote": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "number": "String",
        "type": "String",
        "text": "String",
        "language": "Concept",
    },
    "ClaimResponse_SubDetail": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "subDetailSequence": "String",
        "noteNumber": "List<String>",
        "adjudication": "List<BackboneElement:ClaimResponse_Adjudication>",
    },
    "ClaimResponse_SubDetail1": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "productOrService": "Concept",
        "modifier": "List<Concept>",
        "quantity": "Decimal",
        "unitPrice": "BackboneElement:Money",
        "factor": "Decimal",
        "net": "BackboneElement:Money",
        "noteNumber": "List<String>",
        "adjudication": "List<BackboneElement:ClaimResponse_Adjudication>",
    },
    "ClaimResponse_Total": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "category": "Concept",
        "amount": "BackboneElement:Money",
    },
    "Claim_Accident": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "date": "Date",
        "type": "Concept",
        "location": "Choice:String",
    },
    "Claim_CareTeam": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "sequence": "String",
        "provider": "BackboneElement:Reference",
        "responsible": "Boolean",
        "role": "Concept",
        "qualification": "Concept",
    },
    "Claim_Detail": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "sequence": "String",
        "revenue": "Concept",
        "category": "Concept",
        "productOrService": "Concept",
        "modifier": "List<Concept>",
        "programCode": "List<Concept>",
        "quantity": "Decimal",
        "unitPrice": "BackboneElement:Money",
        "factor": "Decimal",
        "net": "BackboneElement:Money",
        "udi": "List<BackboneElement:Reference>",
        "subDetail": "List<BackboneElement:Claim_SubDetail>",
    },
    "Claim_Diagnosis": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "sequence": "String",
        "diagnosis": "Choice:String",
        "type": "List<Concept>",
        "onAdmission": "Concept",
        "packageCode": "Concept",
    },
    "Claim_Insurance": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "sequence": "String",
        "focal": "Boolean",
        "identifier": "BackboneElement:Identifier",
        "coverage": "BackboneElement:Reference",
        "businessArrangement": "String",
        "preAuthRef": "List<String>",
        "claimResponse": "BackboneElement:Reference",
    },
    "Claim_Item": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "sequence": "String",
        "careTeamSequence": "List<String>",
        "diagnosisSequence": "List<String>",
        "procedureSequence": "List<String>",
        "informationSequence": "List<String>",
        "revenue": "Concept",
        "category": "Concept",
        "productOrService": "Concept",
        "modifier": "List<Concept>",
        "programCode": "List<Concept>",
        "serviced": "Choice:Interval<DateTime>",
        "location": "Choice:String",
        "quantity": "Decimal",
        "unitPrice": "BackboneElement:Money",
        "factor": "Decimal",
        "net": "BackboneElement:Money",
        "udi": "List<BackboneElement:Reference>",
        "bodySite": "Concept",
        "subSite": "List<Concept>",
        "encounter": "List<BackboneElement:Reference>",
        "detail": "List<BackboneElement:Claim_Detail>",
    },
    "Claim_Payee": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "party": "BackboneElement:Reference",
    },
    "Claim_Procedure": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "sequence": "String",
        "type": "List<Concept>",
        "date": "DateTime",
        "procedure": "Choice:String",
        "udi": "List<BackboneElement:Reference>",
    },
    "Claim_Related": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "claim": "BackboneElement:Reference",
        "relationship": "Concept",
        "reference": "BackboneElement:Identifier",
    },
    "Claim_SubDetail": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "sequence": "String",
        "revenue": "Concept",
        "category": "Concept",
        "productOrService": "Concept",
        "modifier": "List<Concept>",
        "programCode": "List<Concept>",
        "quantity": "Decimal",
        "unitPrice": "BackboneElement:Money",
        "factor": "Decimal",
        "net": "BackboneElement:Money",
        "udi": "List<BackboneElement:Reference>",
    },
    "Claim_SupportingInfo": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "sequence": "String",
        "category": "Concept",
        "code": "Concept",
        "timing": "Choice:Interval<DateTime>",
        "value": "Choice:String",
        "reason": "Concept",
    },
    "ClinicalImpression_Finding": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "itemCodeableConcept": "Concept",
        "itemReference": "BackboneElement:Reference",
        "basis": "String",
    },
    "ClinicalImpression_Investigation": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Concept",
        "item": "List<BackboneElement:Reference>",
    },
    "CodeSystem_Concept": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Code",
        "display": "String",
        "definition": "String",
        "designation": "List<BackboneElement:CodeSystem_Designation>",
        "property": "List<BackboneElement:CodeSystem_Property1>",
        "concept": "List<BackboneElement:CodeSystem_Concept>",
    },
    "CodeSystem_Designation": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "language": "Code",
        "use": "Code",
        "value": "String",
    },
    "CodeSystem_Filter": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Code",
        "description": "String",
        "operator": "List<Code>",
        "value": "String",
    },
    "CodeSystem_Property": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Code",
        "uri": "String",
        "description": "String",
        "type": "String",
    },
    "CodeSystem_Property1": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Code",
        "value": "Choice:String",
    },
    "CodeableConcept": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "coding": "List<Code>",
        "text": "String",
    },
    "Coding": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "system": "String",
        "version": "String",
        "code": "Code",
        "display": "String",
        "userSelected": "Boolean",
    },
    "CommunicationRequest_Payload": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "content": "Choice:String",
    },
    "Communication_Payload": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "content": "Choice:String",
    },
    "CompartmentDefinition_Resource": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Code",
        "param": "List<String>",
        "documentation": "String",
    },
    "Composition_Attester": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "mode": "String",
        "time": "DateTime",
        "party": "BackboneElement:Reference",
    },
    "Composition_Event": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "List<Concept>",
        "period": "Interval<DateTime>",
        "detail": "List<BackboneElement:Reference>",
    },
    "Composition_RelatesTo": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Code",
        "target": "Choice:String",
    },
    "Composition_Section": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "title": "String",
        "code": "Concept",
        "author": "List<BackboneElement:Reference>",
        "focus": "BackboneElement:Reference",
        "text": "BackboneElement:Narrative",
        "mode": "Code",
        "orderedBy": "Concept",
        "entry": "List<BackboneElement:Reference>",
        "emptyReason": "Concept",
        "section": "List<BackboneElement:Composition_Section>",
    },
    "ConceptMap_DependsOn": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "property": "String",
        "system": "String",
        "value": "String",
        "display": "String",
    },
    "ConceptMap_Element": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Code",
        "display": "String",
        "target": "List<BackboneElement:ConceptMap_Target>",
    },
    "ConceptMap_Group": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "source": "String",
        "": "Choice:String",
        "target": "String",
        "element": "List<BackboneElement:ConceptMap_Element>",
        "unmapped": "BackboneElement:ConceptMap_Unmapped",
    },
    "ConceptMap_Target": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Code",
        "display": "String",
        "equivalence": "String",
        "comment": "String",
        "dependsOn": "List<BackboneElement:ConceptMap_DependsOn>",
        "product": "List<BackboneElement:ConceptMap_DependsOn>",
    },
    "ConceptMap_Unmapped": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "mode": "String",
        "code": "Code",
        "display": "String",
        "url": "String",
    },
    "Condition_Evidence": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "List<Concept>",
        "detail": "List<BackboneElement:Reference>",
    },
    "Condition_Stage": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "summary": "Concept",
        "assessment": "List<BackboneElement:Reference>",
        "type": "Concept",
    },
    "Consent_Actor": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "role": "Concept",
        "reference": "BackboneElement:Reference",
    },
    "Consent_Data": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "meaning": "String",
        "reference": "BackboneElement:Reference",
    },
    "Consent_Policy": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "authority": "String",
        "uri": "String",
    },
    "Consent_Provision": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "String",
        "period": "Interval<DateTime>",
        "actor": "List<BackboneElement:Consent_Actor>",
        "action": "List<Concept>",
        "securityLabel": "List<Code>",
        "purpose": "List<Code>",
        "class": "List<Code>",
        "code": "List<Concept>",
        "dataPeriod": "Interval<DateTime>",
        "data": "List<BackboneElement:Consent_Data>",
        "provision": "List<BackboneElement:Consent_Provision>",
    },
    "Consent_Verification": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "verified": "Boolean",
        "verifiedWith": "BackboneElement:Reference",
        "verificationDate": "DateTime",
    },
    "ContactDetail": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "name": "String",
        "telecom": "List<BackboneElement:ContactPoint>",
    },
    "ContactPoint": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "system": "String",
        "value": "String",
        "use": "String",
        "rank": "String",
        "period": "Interval<DateTime>",
    },
    "Contract_Action": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "doNotPerform": "Boolean",
        "type": "Concept",
        "subject": "List<BackboneElement:Contract_Subject>",
        "intent": "Concept",
        "linkId": "List<String>",
        "status": "Concept",
        "context": "BackboneElement:Reference",
        "": "Choice:String",
        "occurrence": "Choice:Interval<DateTime>",
        "requester": "List<BackboneElement:Reference>",
        "performerType": "List<Concept>",
        "performerRole": "Concept",
        "performer": "BackboneElement:Reference",
        "performerLinkId": "List<String>",
        "reasonCode": "List<Concept>",
        "reasonReference": "List<BackboneElement:Reference>",
        "reason": "List<String>",
        "reasonLinkId": "List<String>",
        "note": "List<BackboneElement:Annotation>",
        "securityLabelNumber": "List<String>",
    },
    "Contract_Answer": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "value": "Choice:String",
    },
    "Contract_Asset": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "scope": "Concept",
        "type": "List<Concept>",
        "typeReference": "List<BackboneElement:Reference>",
        "subtype": "List<Concept>",
        "relationship": "Code",
        "context": "List<BackboneElement:Contract_Context>",
        "condition": "String",
        "periodType": "List<Concept>",
        "period": "List<Interval<DateTime>>",
        "usePeriod": "List<Interval<DateTime>>",
        "text": "String",
        "linkId": "List<String>",
        "answer": "List<BackboneElement:Contract_Answer>",
        "securityLabelNumber": "List<String>",
        "valuedItem": "List<BackboneElement:Contract_ValuedItem>",
    },
    "Contract_ContentDefinition": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "subType": "Concept",
        "publisher": "BackboneElement:Reference",
        "publicationDate": "DateTime",
        "publicationStatus": "Code",
        "copyright": "String",
    },
    "Contract_Context": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "reference": "BackboneElement:Reference",
        "code": "List<Concept>",
        "text": "String",
    },
    "Contract_Friendly": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "content": "Choice:String",
    },
    "Contract_Legal": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "content": "Choice:String",
    },
    "Contract_Offer": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "party": "List<BackboneElement:Contract_Party>",
        "topic": "BackboneElement:Reference",
        "type": "Concept",
        "decision": "Concept",
        "decisionMode": "List<Concept>",
        "answer": "List<BackboneElement:Contract_Answer>",
        "text": "String",
        "linkId": "List<String>",
        "securityLabelNumber": "List<String>",
    },
    "Contract_Party": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "reference": "List<BackboneElement:Reference>",
        "role": "Concept",
    },
    "Contract_Rule": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "content": "Choice:String",
    },
    "Contract_SecurityLabel": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "number": "List<String>",
        "classification": "Code",
        "category": "List<Code>",
        "control": "List<Code>",
    },
    "Contract_Signer": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Code",
        "party": "BackboneElement:Reference",
        "signature": "List<BackboneElement:Signature>",
    },
    "Contract_Subject": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "reference": "List<BackboneElement:Reference>",
        "role": "Concept",
    },
    "Contract_Term": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "BackboneElement:Identifier",
        "issued": "DateTime",
        "applies": "Interval<DateTime>",
        "topic": "Choice:String",
        "type": "Concept",
        "subType": "Concept",
        "text": "String",
        "securityLabel": "List<BackboneElement:Contract_SecurityLabel>",
        "offer": "BackboneElement:Contract_Offer",
        "asset": "List<BackboneElement:Contract_Asset>",
        "action": "List<BackboneElement:Contract_Action>",
        "group": "List<BackboneElement:Contract_Term>",
    },
    "Contract_ValuedItem": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "entity": "Choice:String",
        "identifier": "BackboneElement:Identifier",
        "effectiveTime": "DateTime",
        "quantity": "Decimal",
        "unitPrice": "BackboneElement:Money",
        "factor": "Decimal",
        "points": "Decimal",
        "net": "BackboneElement:Money",
        "payment": "String",
        "paymentDate": "DateTime",
        "responsible": "BackboneElement:Reference",
        "recipient": "BackboneElement:Reference",
        "linkId": "List<String>",
        "securityLabelNumber": "List<String>",
    },
    "Contributor": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "type": "String",
        "name": "String",
        "contact": "List<BackboneElement:ContactDetail>",
    },
    "Count": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "value": "Decimal",
        "comparator": "String",
        "unit": "String",
        "system": "String",
        "code": "Code",
    },
    "CoverageEligibilityRequest_Diagnosis": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "diagnosis": "Choice:String",
    },
    "CoverageEligibilityRequest_Insurance": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "focal": "Boolean",
        "coverage": "BackboneElement:Reference",
        "businessArrangement": "String",
    },
    "CoverageEligibilityRequest_Item": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "supportingInfoSequence": "List<String>",
        "category": "Concept",
        "productOrService": "Concept",
        "modifier": "List<Concept>",
        "provider": "BackboneElement:Reference",
        "quantity": "Decimal",
        "unitPrice": "BackboneElement:Money",
        "facility": "BackboneElement:Reference",
        "diagnosis": "List<BackboneElement:CoverageEligibilityRequest_Diagnosis>",
        "detail": "List<BackboneElement:Reference>",
    },
    "CoverageEligibilityRequest_SupportingInfo": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "sequence": "String",
        "information": "BackboneElement:Reference",
        "appliesToAll": "Boolean",
    },
    "CoverageEligibilityResponse_Benefit": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "allowed": "Choice:String",
        "used": "Choice:String",
    },
    "CoverageEligibilityResponse_Error": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Concept",
    },
    "CoverageEligibilityResponse_Insurance": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "coverage": "BackboneElement:Reference",
        "inforce": "Boolean",
        "benefitPeriod": "Interval<DateTime>",
        "item": "List<BackboneElement:CoverageEligibilityResponse_Item>",
    },
    "CoverageEligibilityResponse_Item": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "category": "Concept",
        "productOrService": "Concept",
        "modifier": "List<Concept>",
        "provider": "BackboneElement:Reference",
        "excluded": "Boolean",
        "name": "String",
        "description": "String",
        "network": "Concept",
        "unit": "Concept",
        "term": "Concept",
        "benefit": "List<BackboneElement:CoverageEligibilityResponse_Benefit>",
        "authorizationRequired": "Boolean",
        "authorizationSupporting": "List<Concept>",
        "authorizationUrl": "String",
    },
    "Coverage_Class": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "value": "String",
        "name": "String",
    },
    "Coverage_CostToBeneficiary": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "value": "Choice:String",
        "exception": "List<BackboneElement:Coverage_Exception>",
    },
    "Coverage_Exception": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "period": "Interval<DateTime>",
    },
    "DataRequirement": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "type": "Code",
        "profile": "List<String>",
        "subject": "Choice:String",
        "mustSupport": "List<String>",
        "codeFilter": "List<BackboneElement:DataRequirement_CodeFilter>",
        "dateFilter": "List<BackboneElement:DataRequirement_DateFilter>",
        "limit": "String",
        "sort": "List<BackboneElement:DataRequirement_Sort>",
    },
    "DataRequirement_CodeFilter": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "path": "String",
        "searchParam": "String",
        "valueSet": "String",
        "code": "List<Code>",
    },
    "DataRequirement_DateFilter": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "path": "String",
        "searchParam": "String",
        "value": "Choice:Interval<DateTime>",
    },
    "DataRequirement_Sort": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "path": "String",
        "direction": "String",
    },
    "DetectedIssue_Evidence": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "List<Concept>",
        "detail": "List<BackboneElement:Reference>",
    },
    "DetectedIssue_Mitigation": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "action": "Concept",
        "date": "DateTime",
        "author": "BackboneElement:Reference",
    },
    "DeviceDefinition_Capability": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "description": "List<Concept>",
    },
    "DeviceDefinition_DeviceName": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "name": "String",
        "type": "String",
    },
    "DeviceDefinition_Material": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "substance": "Concept",
        "alternate": "Boolean",
        "allergenicIndicator": "Boolean",
    },
    "DeviceDefinition_Property": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "valueQuantity": "List<Decimal>",
        "valueCode": "List<Concept>",
    },
    "DeviceDefinition_Specialization": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "systemType": "String",
        "version": "String",
    },
    "DeviceDefinition_UdiDeviceIdentifier": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "deviceIdentifier": "String",
        "issuer": "String",
        "jurisdiction": "String",
    },
    "DeviceMetric_Calibration": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "String",
        "state": "String",
        "time": "String",
    },
    "DeviceRequest_Parameter": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Concept",
        "value": "Choice:String",
    },
    "Device_DeviceName": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "name": "String",
        "type": "String",
    },
    "Device_Property": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "valueQuantity": "List<Decimal>",
        "valueCode": "List<Concept>",
    },
    "Device_Specialization": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "systemType": "Concept",
        "version": "String",
    },
    "Device_UdiCarrier": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "deviceIdentifier": "String",
        "issuer": "String",
        "jurisdiction": "String",
        "carrierAIDC": "String",
        "carrierHRF": "String",
        "entryType": "String",
    },
    "Device_Version": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "component": "BackboneElement:Identifier",
        "value": "String",
    },
    "DiagnosticReport_Media": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "comment": "String",
        "link": "BackboneElement:Reference",
    },
    "Distance": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "value": "Decimal",
        "comparator": "String",
        "unit": "String",
        "system": "String",
        "code": "Code",
    },
    "DocumentManifest_Related": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "BackboneElement:Identifier",
        "ref": "BackboneElement:Reference",
    },
    "DocumentReference_Content": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "attachment": "BackboneElement:Attachment",
        "format": "Code",
    },
    "DocumentReference_Context": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "encounter": "List<BackboneElement:Reference>",
        "event": "List<Concept>",
        "period": "Interval<DateTime>",
        "facilityType": "Concept",
        "practiceSetting": "Concept",
        "sourcePatientInfo": "BackboneElement:Reference",
        "related": "List<BackboneElement:Reference>",
    },
    "DocumentReference_RelatesTo": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "String",
        "target": "BackboneElement:Reference",
    },
    "Dosage": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "sequence": "Integer",
        "text": "String",
        "additionalInstruction": "List<Concept>",
        "patientInstruction": "String",
        "timing": "BackboneElement:Timing",
        "asNeeded": "Choice:String",
        "site": "Concept",
        "route": "Concept",
        "method": "Concept",
        "doseAndRate": "List<BackboneElement:Dosage_DoseAndRate>",
        "maxDosePerPeriod": "BackboneElement:Ratio",
        "maxDosePerAdministration": "Decimal",
        "maxDosePerLifetime": "Decimal",
    },
    "Dosage_DoseAndRate": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "dose": "Choice:String",
        "rate": "Choice:String",
    },
    "Duration": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "value": "Decimal",
        "comparator": "String",
        "unit": "String",
        "system": "String",
        "code": "Code",
    },
    "EffectEvidenceSynthesis_Certainty": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "rating": "List<Concept>",
        "note": "List<BackboneElement:Annotation>",
        "certaintySubcomponent": "List<BackboneElement:EffectEvidenceSynthesis_CertaintySubcomponent>",
    },
    "EffectEvidenceSynthesis_CertaintySubcomponent": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "rating": "List<Concept>",
        "note": "List<BackboneElement:Annotation>",
    },
    "EffectEvidenceSynthesis_EffectEstimate": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "description": "String",
        "type": "Concept",
        "variantState": "Concept",
        "value": "Decimal",
        "unitOfMeasure": "Concept",
        "precisionEstimate": "List<BackboneElement:EffectEvidenceSynthesis_PrecisionEstimate>",
    },
    "EffectEvidenceSynthesis_PrecisionEstimate": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "level": "Decimal",
        "from": "Decimal",
        "to": "Decimal",
    },
    "EffectEvidenceSynthesis_ResultsByExposure": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "description": "String",
        "exposureState": "String",
        "variantState": "Concept",
        "riskEvidenceSynthesis": "BackboneElement:Reference",
    },
    "EffectEvidenceSynthesis_SampleSize": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "description": "String",
        "numberOfStudies": "Integer",
        "numberOfParticipants": "Integer",
    },
    "Element": {"id": "String", "extension": "List<BackboneElement:Extension>"},
    "ElementDefinition": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "path": "String",
        "representation": "List<String>",
        "sliceName": "String",
        "sliceIsConstraining": "Boolean",
        "label": "String",
        "code": "List<Code>",
        "slicing": "BackboneElement:ElementDefinition_Slicing",
        "short": "String",
        "definition": "String",
        "comment": "String",
        "requirements": "String",
        "alias": "List<String>",
        "min": "String",
        "max": "String",
        "base": "BackboneElement:ElementDefinition_Base",
        "contentReference": "String",
        "type": "List<BackboneElement:ElementDefinition_Type>",
        "defaultValue": "Choice:Interval<DateTime>",
        "meaningWhenMissing": "String",
        "orderMeaning": "String",
        "fixed": "Choice:Interval<DateTime>",
        "pattern": "Choice:Interval<DateTime>",
        "example": "List<BackboneElement:ElementDefinition_Example>",
        "minValue": "Choice:String",
        "maxValue": "Choice:String",
        "maxLength": "Integer",
        "condition": "List<String>",
        "constraint": "List<BackboneElement:ElementDefinition_Constraint>",
        "mustSupport": "Boolean",
        "isModifier": "Boolean",
        "isModifierReason": "String",
        "isSummary": "Boolean",
        "binding": "BackboneElement:ElementDefinition_Binding",
        "mapping": "List<BackboneElement:ElementDefinition_Mapping>",
    },
    "ElementDefinition_Base": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "path": "String",
        "min": "String",
        "max": "String",
    },
    "ElementDefinition_Binding": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "strength": "String",
        "description": "String",
        "valueSet": "String",
    },
    "ElementDefinition_Constraint": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "key": "String",
        "requirements": "String",
        "severity": "String",
        "human": "String",
        "expression": "String",
        "xpath": "String",
        "source": "String",
    },
    "ElementDefinition_Discriminator": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "String",
        "path": "String",
    },
    "ElementDefinition_Example": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "label": "String",
        "value": "Choice:Interval<DateTime>",
    },
    "ElementDefinition_Mapping": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identity": "String",
        "language": "Code",
        "map": "String",
        "comment": "String",
    },
    "ElementDefinition_Slicing": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "discriminator": "List<BackboneElement:ElementDefinition_Discriminator>",
        "description": "String",
        "ordered": "Boolean",
        "rules": "String",
    },
    "ElementDefinition_Type": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "String",
        "profile": "List<String>",
        "targetProfile": "List<String>",
        "aggregation": "List<String>",
        "versioning": "String",
    },
    "Encounter_ClassHistory": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "class": "Code",
        "period": "Interval<DateTime>",
    },
    "Encounter_Diagnosis": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "condition": "BackboneElement:Reference",
        "use": "Concept",
        "rank": "String",
    },
    "Encounter_Hospitalization": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "preAdmissionIdentifier": "BackboneElement:Identifier",
        "origin": "BackboneElement:Reference",
        "admitSource": "Concept",
        "reAdmission": "Concept",
        "dietPreference": "List<Concept>",
        "specialCourtesy": "List<Concept>",
        "specialArrangement": "List<Concept>",
        "destination": "BackboneElement:Reference",
        "dischargeDisposition": "Concept",
    },
    "Encounter_Location": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "location": "BackboneElement:Reference",
        "status": "String",
        "physicalType": "Concept",
        "period": "Interval<DateTime>",
    },
    "Encounter_Participant": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "List<Concept>",
        "period": "Interval<DateTime>",
        "individual": "BackboneElement:Reference",
    },
    "Encounter_StatusHistory": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "status": "String",
        "period": "Interval<DateTime>",
    },
    "EpisodeOfCare_Diagnosis": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "condition": "BackboneElement:Reference",
        "role": "Concept",
        "rank": "String",
    },
    "EpisodeOfCare_StatusHistory": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "status": "String",
        "period": "Interval<DateTime>",
    },
    "EvidenceVariable_Characteristic": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "description": "String",
        "definition": "Choice:String",
        "usageContext": "List<BackboneElement:UsageContext>",
        "exclude": "Boolean",
        "participantEffective": "Choice:Interval<DateTime>",
        "timeFromStart": "BackboneElement:Duration",
        "groupMeasure": "String",
    },
    "ExampleScenario_Actor": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "actorId": "String",
        "type": "String",
        "name": "String",
        "description": "String",
    },
    "ExampleScenario_Alternative": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "title": "String",
        "description": "String",
        "step": "List<BackboneElement:ExampleScenario_Step>",
    },
    "ExampleScenario_ContainedInstance": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "resourceId": "String",
        "versionId": "String",
    },
    "ExampleScenario_Instance": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "resourceId": "String",
        "resourceType": "Code",
        "name": "String",
        "description": "String",
        "version": "List<BackboneElement:ExampleScenario_Version>",
        "containedInstance": "List<BackboneElement:ExampleScenario_ContainedInstance>",
    },
    "ExampleScenario_Operation": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "number": "String",
        "type": "String",
        "name": "String",
        "initiator": "String",
        "receiver": "String",
        "description": "String",
        "initiatorActive": "Boolean",
        "receiverActive": "Boolean",
        "request": "BackboneElement:ExampleScenario_ContainedInstance",
        "response": "BackboneElement:ExampleScenario_ContainedInstance",
    },
    "ExampleScenario_Process": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "title": "String",
        "description": "String",
        "preConditions": "String",
        "postConditions": "String",
        "step": "List<BackboneElement:ExampleScenario_Step>",
    },
    "ExampleScenario_Step": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "process": "List<BackboneElement:ExampleScenario_Process>",
        "pause": "Boolean",
        "operation": "BackboneElement:ExampleScenario_Operation",
        "alternative": "List<BackboneElement:ExampleScenario_Alternative>",
    },
    "ExampleScenario_Version": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "versionId": "String",
        "description": "String",
    },
    "ExplanationOfBenefit_Accident": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "date": "Date",
        "type": "Concept",
        "location": "Choice:String",
    },
    "ExplanationOfBenefit_AddItem": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "itemSequence": "List<String>",
        "detailSequence": "List<String>",
        "subDetailSequence": "List<String>",
        "provider": "List<BackboneElement:Reference>",
        "productOrService": "Concept",
        "modifier": "List<Concept>",
        "programCode": "List<Concept>",
        "serviced": "Choice:Interval<DateTime>",
        "location": "Choice:String",
        "quantity": "Decimal",
        "unitPrice": "BackboneElement:Money",
        "factor": "Decimal",
        "net": "BackboneElement:Money",
        "bodySite": "Concept",
        "subSite": "List<Concept>",
        "noteNumber": "List<String>",
        "adjudication": "List<BackboneElement:ExplanationOfBenefit_Adjudication>",
        "detail": "List<BackboneElement:ExplanationOfBenefit_Detail1>",
    },
    "ExplanationOfBenefit_Adjudication": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "category": "Concept",
        "reason": "Concept",
        "amount": "BackboneElement:Money",
        "value": "Decimal",
    },
    "ExplanationOfBenefit_BenefitBalance": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "category": "Concept",
        "excluded": "Boolean",
        "name": "String",
        "description": "String",
        "network": "Concept",
        "unit": "Concept",
        "term": "Concept",
        "financial": "List<BackboneElement:ExplanationOfBenefit_Financial>",
    },
    "ExplanationOfBenefit_CareTeam": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "sequence": "String",
        "provider": "BackboneElement:Reference",
        "responsible": "Boolean",
        "role": "Concept",
        "qualification": "Concept",
    },
    "ExplanationOfBenefit_Detail": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "sequence": "String",
        "revenue": "Concept",
        "category": "Concept",
        "productOrService": "Concept",
        "modifier": "List<Concept>",
        "programCode": "List<Concept>",
        "quantity": "Decimal",
        "unitPrice": "BackboneElement:Money",
        "factor": "Decimal",
        "net": "BackboneElement:Money",
        "udi": "List<BackboneElement:Reference>",
        "noteNumber": "List<String>",
        "adjudication": "List<BackboneElement:ExplanationOfBenefit_Adjudication>",
        "subDetail": "List<BackboneElement:ExplanationOfBenefit_SubDetail>",
    },
    "ExplanationOfBenefit_Detail1": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "productOrService": "Concept",
        "modifier": "List<Concept>",
        "quantity": "Decimal",
        "unitPrice": "BackboneElement:Money",
        "factor": "Decimal",
        "net": "BackboneElement:Money",
        "noteNumber": "List<String>",
        "adjudication": "List<BackboneElement:ExplanationOfBenefit_Adjudication>",
        "subDetail": "List<BackboneElement:ExplanationOfBenefit_SubDetail1>",
    },
    "ExplanationOfBenefit_Diagnosis": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "sequence": "String",
        "diagnosis": "Choice:String",
        "type": "List<Concept>",
        "onAdmission": "Concept",
        "packageCode": "Concept",
    },
    "ExplanationOfBenefit_Financial": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "allowed": "Choice:String",
        "used": "Choice:String",
    },
    "ExplanationOfBenefit_Insurance": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "focal": "Boolean",
        "coverage": "BackboneElement:Reference",
        "preAuthRef": "List<String>",
    },
    "ExplanationOfBenefit_Item": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "sequence": "String",
        "careTeamSequence": "List<String>",
        "diagnosisSequence": "List<String>",
        "procedureSequence": "List<String>",
        "informationSequence": "List<String>",
        "revenue": "Concept",
        "category": "Concept",
        "productOrService": "Concept",
        "modifier": "List<Concept>",
        "programCode": "List<Concept>",
        "serviced": "Choice:Interval<DateTime>",
        "location": "Choice:String",
        "quantity": "Decimal",
        "unitPrice": "BackboneElement:Money",
        "factor": "Decimal",
        "net": "BackboneElement:Money",
        "udi": "List<BackboneElement:Reference>",
        "bodySite": "Concept",
        "subSite": "List<Concept>",
        "encounter": "List<BackboneElement:Reference>",
        "noteNumber": "List<String>",
        "adjudication": "List<BackboneElement:ExplanationOfBenefit_Adjudication>",
        "detail": "List<BackboneElement:ExplanationOfBenefit_Detail>",
    },
    "ExplanationOfBenefit_Payee": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "party": "BackboneElement:Reference",
    },
    "ExplanationOfBenefit_Payment": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "adjustment": "BackboneElement:Money",
        "adjustmentReason": "Concept",
        "date": "Date",
        "amount": "BackboneElement:Money",
        "identifier": "BackboneElement:Identifier",
    },
    "ExplanationOfBenefit_Procedure": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "sequence": "String",
        "type": "List<Concept>",
        "date": "DateTime",
        "procedure": "Choice:String",
        "udi": "List<BackboneElement:Reference>",
    },
    "ExplanationOfBenefit_ProcessNote": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "number": "String",
        "type": "String",
        "text": "String",
        "language": "Concept",
    },
    "ExplanationOfBenefit_Related": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "claim": "BackboneElement:Reference",
        "relationship": "Concept",
        "reference": "BackboneElement:Identifier",
    },
    "ExplanationOfBenefit_SubDetail": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "sequence": "String",
        "revenue": "Concept",
        "category": "Concept",
        "productOrService": "Concept",
        "modifier": "List<Concept>",
        "programCode": "List<Concept>",
        "quantity": "Decimal",
        "unitPrice": "BackboneElement:Money",
        "factor": "Decimal",
        "net": "BackboneElement:Money",
        "udi": "List<BackboneElement:Reference>",
        "noteNumber": "List<String>",
        "adjudication": "List<BackboneElement:ExplanationOfBenefit_Adjudication>",
    },
    "ExplanationOfBenefit_SubDetail1": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "productOrService": "Concept",
        "modifier": "List<Concept>",
        "quantity": "Decimal",
        "unitPrice": "BackboneElement:Money",
        "factor": "Decimal",
        "net": "BackboneElement:Money",
        "noteNumber": "List<String>",
        "adjudication": "List<BackboneElement:ExplanationOfBenefit_Adjudication>",
    },
    "ExplanationOfBenefit_SupportingInfo": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "sequence": "String",
        "category": "Concept",
        "code": "Concept",
        "timing": "Choice:Interval<DateTime>",
        "value": "Choice:String",
        "reason": "Code",
    },
    "ExplanationOfBenefit_Total": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "category": "Concept",
        "amount": "BackboneElement:Money",
    },
    "Expression": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "description": "String",
        "name": "String",
        "language": "String",
        "expression": "String",
        "reference": "String",
    },
    "Extension": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "url": "String",
        "value": "Choice:Interval<DateTime>",
    },
    "FamilyMemberHistory_Condition": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Concept",
        "outcome": "Concept",
        "contributedToDeath": "Boolean",
        "onset": "Choice:Interval<DateTime>",
        "note": "List<BackboneElement:Annotation>",
    },
    "Goal_Target": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "measure": "Concept",
        "detail": "Choice:String",
        "dueD": "Choice:String",
    },
    "GraphDefinition_Compartment": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "use": "String",
        "code": "Code",
        "rule": "String",
        "expression": "String",
        "description": "String",
    },
    "GraphDefinition_Link": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "path": "String",
        "sliceName": "String",
        "min": "Integer",
        "max": "String",
        "description": "String",
        "target": "List<BackboneElement:GraphDefinition_Target>",
    },
    "GraphDefinition_Target": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Code",
        "params": "String",
        "profile": "String",
        "compartment": "List<BackboneElement:GraphDefinition_Compartment>",
        "link": "List<BackboneElement:GraphDefinition_Link>",
    },
    "Group_Characteristic": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Concept",
        "value": "Choice:String",
        "exclude": "Boolean",
        "period": "Interval<DateTime>",
    },
    "Group_Member": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "entity": "BackboneElement:Reference",
        "period": "Interval<DateTime>",
        "inactive": "Boolean",
    },
    "HealthcareService_AvailableTime": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "daysOfWeek": "List<String>",
        "allDay": "Boolean",
        "availableStartTime": "String",
        "availableEndTime": "String",
    },
    "HealthcareService_Eligibility": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Concept",
        "comment": "String",
    },
    "HealthcareService_NotAvailable": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "description": "String",
        "during": "Interval<DateTime>",
    },
    "HumanName": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "use": "String",
        "text": "String",
        "family": "String",
        "given": "List<String>",
        "prefix": "List<String>",
        "suffix": "List<String>",
        "period": "Interval<DateTime>",
    },
    "Identifier": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "use": "String",
        "type": "Concept",
        "system": "String",
        "value": "String",
        "period": "Interval<DateTime>",
        "assigner": "BackboneElement:Reference",
    },
    "ImagingStudy_Instance": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "uid": "String",
        "sopClass": "Code",
        "number": "String",
        "title": "String",
    },
    "ImagingStudy_Performer": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "function": "Concept",
        "actor": "BackboneElement:Reference",
    },
    "ImagingStudy_Series": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "uid": "String",
        "number": "String",
        "modality": "Code",
        "description": "String",
        "numberOfInstances": "String",
        "endpoint": "List<BackboneElement:Reference>",
        "bodySite": "Code",
        "laterality": "Code",
        "specimen": "List<BackboneElement:Reference>",
        "started": "DateTime",
        "performer": "List<BackboneElement:ImagingStudy_Performer>",
        "instance": "List<BackboneElement:ImagingStudy_Instance>",
    },
    "ImmunizationRecommendation_DateCriterion": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Concept",
        "value": "DateTime",
    },
    "ImmunizationRecommendation_Recommendation": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "vaccineCode": "List<Concept>",
        "targetDisease": "Concept",
        "contraindicatedVaccineCode": "List<Concept>",
        "forecastStatus": "Concept",
        "forecastReason": "List<Concept>",
        "dateCriterion": "List<BackboneElement:ImmunizationRecommendation_DateCriterion>",
        "description": "String",
        "series": "String",
        "doseNumber": "Choice:String",
        "seriesDoses": "Choice:String",
        "supportingImmunization": "List<BackboneElement:Reference>",
        "supportingPatientInformation": "List<BackboneElement:Reference>",
    },
    "Immunization_Education": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "documentType": "String",
        "reference": "String",
        "publicationDate": "DateTime",
        "presentationDate": "DateTime",
    },
    "Immunization_Performer": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "function": "Concept",
        "actor": "BackboneElement:Reference",
    },
    "Immunization_ProtocolApplied": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "series": "String",
        "authority": "BackboneElement:Reference",
        "targetDisease": "List<Concept>",
        "doseNumber": "Choice:String",
        "seriesDoses": "Choice:String",
    },
    "Immunization_Reaction": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "date": "DateTime",
        "detail": "BackboneElement:Reference",
        "reported": "Boolean",
    },
    "ImplementationGuide_Definition": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "grouping": "List<BackboneElement:ImplementationGuide_Grouping>",
        "resource": "List<BackboneElement:ImplementationGuide_Resource>",
        "page": "BackboneElement:ImplementationGuide_Page",
        "parameter": "List<BackboneElement:ImplementationGuide_Parameter>",
        "template": "List<BackboneElement:ImplementationGuide_Template>",
    },
    "ImplementationGuide_DependsOn": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "uri": "String",
        "packageId": "String",
        "version": "String",
    },
    "ImplementationGuide_Global": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Code",
        "profile": "String",
    },
    "ImplementationGuide_Grouping": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "name": "String",
        "description": "String",
    },
    "ImplementationGuide_Manifest": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "rendering": "String",
        "resource": "List<BackboneElement:ImplementationGuide_Resource1>",
        "page": "List<BackboneElement:ImplementationGuide_Page1>",
        "image": "List<String>",
        "other": "List<String>",
    },
    "ImplementationGuide_Page": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "name": "Choice:String",
        "title": "String",
        "generation": "String",
        "page": "List<BackboneElement:ImplementationGuide_Page>",
    },
    "ImplementationGuide_Page1": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "name": "String",
        "title": "String",
        "anchor": "List<String>",
    },
    "ImplementationGuide_Parameter": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "String",
        "value": "String",
    },
    "ImplementationGuide_Resource": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "reference": "BackboneElement:Reference",
        "fhirVersion": "List<String>",
        "name": "String",
        "description": "String",
        "example": "Choice:String",
        "groupingId": "String",
    },
    "ImplementationGuide_Resource1": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "reference": "BackboneElement:Reference",
        "example": "Choice:String",
        "relativePath": "String",
    },
    "ImplementationGuide_Template": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Code",
        "source": "String",
        "scope": "String",
    },
    "InsurancePlan_Benefit": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "requirement": "String",
        "limit": "List<BackboneElement:InsurancePlan_Limit>",
    },
    "InsurancePlan_Benefit1": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "cost": "List<BackboneElement:InsurancePlan_Cost>",
    },
    "InsurancePlan_Contact": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "purpose": "Concept",
        "name": "BackboneElement:HumanName",
        "telecom": "List<BackboneElement:ContactPoint>",
        "address": "BackboneElement:Address",
    },
    "InsurancePlan_Cost": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "applicability": "Concept",
        "qualifiers": "List<Concept>",
        "value": "Decimal",
    },
    "InsurancePlan_Coverage": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "network": "List<BackboneElement:Reference>",
        "benefit": "List<BackboneElement:InsurancePlan_Benefit>",
    },
    "InsurancePlan_GeneralCost": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "groupSize": "String",
        "cost": "BackboneElement:Money",
        "comment": "String",
    },
    "InsurancePlan_Limit": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "value": "Decimal",
        "code": "Concept",
    },
    "InsurancePlan_Plan": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "type": "Concept",
        "coverageArea": "List<BackboneElement:Reference>",
        "network": "List<BackboneElement:Reference>",
        "generalCost": "List<BackboneElement:InsurancePlan_GeneralCost>",
        "specificCost": "List<BackboneElement:InsurancePlan_SpecificCost>",
    },
    "InsurancePlan_SpecificCost": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "category": "Concept",
        "benefit": "List<BackboneElement:InsurancePlan_Benefit1>",
    },
    "Invoice_LineItem": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "sequence": "String",
        "chargeItem": "Choice:String",
        "priceComponent": "List<BackboneElement:Invoice_PriceComponent>",
    },
    "Invoice_Participant": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "role": "Concept",
        "actor": "BackboneElement:Reference",
    },
    "Invoice_PriceComponent": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "String",
        "code": "Concept",
        "factor": "Decimal",
        "amount": "BackboneElement:Money",
    },
    "Linkage_Item": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "String",
        "resource": "BackboneElement:Reference",
    },
    "List_Entry": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "flag": "Concept",
        "deleted": "Boolean",
        "date": "DateTime",
        "item": "BackboneElement:Reference",
    },
    "Location_HoursOfOperation": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "daysOfWeek": "List<Code>",
        "allDay": "Boolean",
        "openingTime": "String",
        "closingTime": "String",
    },
    "Location_Position": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "longitude": "Decimal",
        "latitude": "Decimal",
        "altitude": "Decimal",
    },
    "MarketingStatus": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "country": "Concept",
        "jurisdiction": "Concept",
        "status": "Concept",
        "": "Choice:Interval<DateTime>",
    },
    "MeasureReport_Component": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Concept",
        "value": "Concept",
    },
    "MeasureReport_Group": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Concept",
        "population": "List<BackboneElement:MeasureReport_Population>",
        "measureScore": "Decimal",
        "stratifier": "List<BackboneElement:MeasureReport_Stratifier>",
    },
    "MeasureReport_Population": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Concept",
        "count": "Integer",
        "subjectResults": "BackboneElement:Reference",
    },
    "MeasureReport_Population1": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Concept",
        "count": "Integer",
        "subjectResults": "BackboneElement:Reference",
    },
    "MeasureReport_Stratifier": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "List<Concept>",
        "stratum": "List<BackboneElement:MeasureReport_Stratum>",
    },
    "MeasureReport_Stratum": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "value": "Concept",
        "component": "List<BackboneElement:MeasureReport_Component>",
        "population": "List<BackboneElement:MeasureReport_Population1>",
        "measureScore": "Decimal",
    },
    "Measure_Component": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Concept",
        "description": "String",
        "criteria": "BackboneElement:Expression",
    },
    "Measure_Group": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Concept",
        "description": "String",
        "population": "List<BackboneElement:Measure_Population>",
        "stratifier": "List<BackboneElement:Measure_Stratifier>",
    },
    "Measure_Population": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Concept",
        "description": "String",
        "criteria": "BackboneElement:Expression",
    },
    "Measure_Stratifier": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Concept",
        "description": "String",
        "criteria": "BackboneElement:Expression",
        "component": "List<BackboneElement:Measure_Component>",
    },
    "Measure_SupplementalData": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Concept",
        "usage": "List<Concept>",
        "description": "String",
        "criteria": "BackboneElement:Expression",
    },
    "MedicationAdministration_Dosage": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "text": "String",
        "site": "Concept",
        "route": "Concept",
        "method": "Concept",
        "dose": "Decimal",
        "rate": "Choice:String",
    },
    "MedicationAdministration_Performer": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "function": "Concept",
        "actor": "BackboneElement:Reference",
    },
    "MedicationDispense_Performer": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "function": "Concept",
        "actor": "BackboneElement:Reference",
    },
    "MedicationDispense_Substitution": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "wasSubstituted": "Boolean",
        "type": "Concept",
        "reason": "List<Concept>",
        "responsibleParty": "List<BackboneElement:Reference>",
    },
    "MedicationKnowledge_AdministrationGuidelines": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "dosage": "List<BackboneElement:MedicationKnowledge_Dosage>",
        "indication": "Choice:String",
        "patientCharacteristics": "List<BackboneElement:MedicationKnowledge_PatientCharacteristics>",
    },
    "MedicationKnowledge_Cost": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "source": "String",
        "cost": "BackboneElement:Money",
    },
    "MedicationKnowledge_Dosage": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "dosage": "List<BackboneElement:Dosage>",
    },
    "MedicationKnowledge_DrugCharacteristic": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "value": "Choice:String",
    },
    "MedicationKnowledge_Ingredient": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "item": "Choice:String",
        "isActive": "Boolean",
        "strength": "BackboneElement:Ratio",
    },
    "MedicationKnowledge_Kinetics": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "areaUnderCurve": "List<Decimal>",
        "lethalDose50": "List<Decimal>",
        "halfLifePeriod": "BackboneElement:Duration",
    },
    "MedicationKnowledge_MaxDispense": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "quantity": "Decimal",
        "period": "BackboneElement:Duration",
    },
    "MedicationKnowledge_MedicineClassification": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "classification": "List<Concept>",
    },
    "MedicationKnowledge_MonitoringProgram": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "name": "String",
    },
    "MedicationKnowledge_Monograph": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "source": "BackboneElement:Reference",
    },
    "MedicationKnowledge_Packaging": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "quantity": "Decimal",
    },
    "MedicationKnowledge_PatientCharacteristics": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "characteristic": "Choice:String",
        "value": "List<String>",
    },
    "MedicationKnowledge_Regulatory": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "regulatoryAuthority": "BackboneElement:Reference",
        "substitution": "List<BackboneElement:MedicationKnowledge_Substitution>",
        "schedule": "List<BackboneElement:MedicationKnowledge_Schedule>",
        "maxDispense": "BackboneElement:MedicationKnowledge_MaxDispense",
    },
    "MedicationKnowledge_RelatedMedicationKnowledge": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "reference": "List<BackboneElement:Reference>",
    },
    "MedicationKnowledge_Schedule": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "schedule": "Concept",
    },
    "MedicationKnowledge_Substitution": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "allowed": "Boolean",
    },
    "MedicationRequest_DispenseRequest": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "initialFill": "BackboneElement:MedicationRequest_InitialFill",
        "dispenseInterval": "BackboneElement:Duration",
        "validityPeriod": "Interval<DateTime>",
        "numberOfRepeatsAllowed": "String",
        "quantity": "Decimal",
        "expectedSupplyDuration": "BackboneElement:Duration",
        "performer": "BackboneElement:Reference",
    },
    "MedicationRequest_InitialFill": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "quantity": "Decimal",
        "duration": "BackboneElement:Duration",
    },
    "MedicationRequest_Substitution": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "allowed": "Choice:String",
        "reason": "Concept",
    },
    "Medication_Batch": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "lotNumber": "String",
        "expirationDate": "DateTime",
    },
    "Medication_Ingredient": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "item": "Choice:String",
        "isActive": "Boolean",
        "strength": "BackboneElement:Ratio",
    },
    "MedicinalProductAuthorization_JurisdictionalAuthorization": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "country": "Concept",
        "jurisdiction": "List<Concept>",
        "legalStatusOfSupply": "Concept",
        "validityPeriod": "Interval<DateTime>",
    },
    "MedicinalProductAuthorization_Procedure": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "BackboneElement:Identifier",
        "type": "Concept",
        "date": "Choice:Interval<DateTime>",
        "application": "List<BackboneElement:MedicinalProductAuthorization_Procedure>",
    },
    "MedicinalProductContraindication_OtherTherapy": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "therapyRelationshipType": "Concept",
        "medication": "Choice:String",
    },
    "MedicinalProductIndication_OtherTherapy": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "therapyRelationshipType": "Concept",
        "medication": "Choice:String",
    },
    "MedicinalProductIngredient_ReferenceStrength": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "substance": "Concept",
        "strength": "Choice:String",
        "measurementPoint": "String",
        "country": "List<Concept>",
    },
    "MedicinalProductIngredient_SpecifiedSubstance": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Concept",
        "group": "Concept",
        "confidentiality": "Concept",
        "strength": "List<BackboneElement:MedicinalProductIngredient_Strength>",
    },
    "MedicinalProductIngredient_Strength": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "presentation": "BackboneElement:Ratio",
        "presentationLowLimit": "BackboneElement:Ratio",
        "concentration": "BackboneElement:Ratio",
        "concentrationLowLimit": "BackboneElement:Ratio",
        "measurementPoint": "String",
        "country": "List<Concept>",
        "referenceStrength": "List<BackboneElement:MedicinalProductIngredient_ReferenceStrength>",
    },
    "MedicinalProductIngredient_Substance": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Concept",
        "strength": "List<BackboneElement:MedicinalProductIngredient_Strength>",
    },
    "MedicinalProductInteraction_Interactant": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "item": "Choice:String",
    },
    "MedicinalProductPackaged_BatchIdentifier": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "outerPackaging": "BackboneElement:Identifier",
        "immediatePackaging": "BackboneElement:Identifier",
    },
    "MedicinalProductPackaged_PackageItem": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "type": "Concept",
        "quantity": "Decimal",
        "material": "List<Concept>",
        "alternateMaterial": "List<Concept>",
        "device": "List<BackboneElement:Reference>",
        "manufacturedItem": "List<BackboneElement:Reference>",
        "packageItem": "List<BackboneElement:MedicinalProductPackaged_PackageItem>",
        "physicalCharacteristics": "BackboneElement:ProdCharacteristic",
        "otherCharacteristics": "List<Concept>",
        "shelfLifeStorage": "List<BackboneElement:ProductShelfLife>",
        "manufacturer": "List<BackboneElement:Reference>",
    },
    "MedicinalProductPharmaceutical_Characteristics": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Concept",
        "status": "Concept",
    },
    "MedicinalProductPharmaceutical_RouteOfAdministration": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Concept",
        "firstDose": "Decimal",
        "maxSingleDose": "Decimal",
        "maxDosePerDay": "Decimal",
        "maxDosePerTreatmentPeriod": "BackboneElement:Ratio",
        "maxTreatmentPeriod": "BackboneElement:Duration",
        "targetSpecies": "List<BackboneElement:MedicinalProductPharmaceutical_TargetSpecies>",
    },
    "MedicinalProductPharmaceutical_TargetSpecies": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Concept",
        "withdrawalPeriod": "List<BackboneElement:MedicinalProductPharmaceutical_WithdrawalPeriod>",
    },
    "MedicinalProductPharmaceutical_WithdrawalPeriod": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "tissue": "Concept",
        "value": "Decimal",
        "supportingInformation": "String",
    },
    "MedicinalProduct_CountryLanguage": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "country": "Concept",
        "jurisdiction": "Concept",
        "language": "Concept",
    },
    "MedicinalProduct_ManufacturingBusinessOperation": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "operationType": "Concept",
        "authorisationReferenceNumber": "BackboneElement:Identifier",
        "effectiveDate": "DateTime",
        "confidentialityIndicator": "Concept",
        "manufacturer": "List<BackboneElement:Reference>",
        "regulator": "BackboneElement:Reference",
    },
    "MedicinalProduct_Name": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "productName": "String",
        "namePart": "List<BackboneElement:MedicinalProduct_NamePart>",
        "countryLanguage": "List<BackboneElement:MedicinalProduct_CountryLanguage>",
    },
    "MedicinalProduct_NamePart": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "part": "String",
        "type": "Code",
    },
    "MedicinalProduct_SpecialDesignation": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "type": "Concept",
        "intendedUse": "Concept",
        "indication": "Choice:String",
        "status": "Concept",
        "date": "DateTime",
        "species": "Concept",
    },
    "MessageDefinition_AllowedResponse": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "message": "String",
        "situation": "String",
    },
    "MessageDefinition_Focus": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Code",
        "profile": "String",
        "min": "String",
        "max": "String",
    },
    "MessageHeader_Destination": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "name": "String",
        "target": "BackboneElement:Reference",
        "endpoint": "String",
        "receiver": "BackboneElement:Reference",
    },
    "MessageHeader_Response": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "String",
        "code": "String",
        "details": "BackboneElement:Reference",
    },
    "MessageHeader_Source": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "name": "String",
        "software": "String",
        "version": "String",
        "contact": "BackboneElement:ContactPoint",
        "endpoint": "String",
    },
    "Meta": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "versionId": "String",
        "lastUpdated": "String",
        "source": "String",
        "profile": "List<String>",
        "security": "List<Code>",
        "tag": "List<Code>",
    },
    "MolecularSequence_Inner": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "start": "Integer",
        "end": "Integer",
    },
    "MolecularSequence_Outer": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "start": "Integer",
        "end": "Integer",
    },
    "MolecularSequence_Quality": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "String",
        "standardSequence": "Concept",
        "start": "Integer",
        "end": "Integer",
        "score": "Decimal",
        "method": "Concept",
        "truthTP": "Decimal",
        "queryTP": "Decimal",
        "truthFN": "Decimal",
        "queryFP": "Decimal",
        "gtFP": "Decimal",
        "precision": "Decimal",
        "recall": "Decimal",
        "fScore": "Decimal",
        "roc": "BackboneElement:MolecularSequence_Roc",
    },
    "MolecularSequence_ReferenceSeq": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "chromosome": "Concept",
        "genomeBuild": "String",
        "orientation": "String",
        "referenceSeqId": "Concept",
        "referenceSeqPointer": "BackboneElement:Reference",
        "referenceSeqString": "String",
        "strand": "String",
        "windowStart": "Integer",
        "windowEnd": "Integer",
    },
    "MolecularSequence_Repository": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "String",
        "": "Choice:String",
        "datasetId": "String",
        "variantsetId": "String",
        "readsetId": "String",
    },
    "MolecularSequence_Roc": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "score": "List<Integer>",
        "numTP": "List<Integer>",
        "numFP": "List<Integer>",
        "numFN": "List<Integer>",
        "precision": "List<Decimal>",
        "sensitivity": "List<Decimal>",
        "fMeasure": "List<Decimal>",
    },
    "MolecularSequence_StructureVariant": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "variantType": "Concept",
        "exact": "Boolean",
        "length": "Integer",
        "outer": "BackboneElement:MolecularSequence_Outer",
        "inner": "BackboneElement:MolecularSequence_Inner",
    },
    "MolecularSequence_Variant": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "start": "Integer",
        "end": "Integer",
        "observedAllele": "String",
        "referenceAllele": "String",
        "cigar": "String",
        "variantPointer": "BackboneElement:Reference",
    },
    "Money": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "value": "Decimal",
        "currency": "Code",
    },
    "NamingSystem_UniqueId": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "String",
        "value": "String",
        "preferred": "Boolean",
        "comment": "String",
        "period": "Interval<DateTime>",
    },
    "Narrative": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "status": "String",
        "div": "String",
    },
    "NutritionOrder_Administration": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "schedule": "BackboneElement:Timing",
        "quantity": "Decimal",
        "rate": "Choice:String",
    },
    "NutritionOrder_EnteralFormula": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "baseFormulaType": "Concept",
        "baseFormulaProductName": "String",
        "additiveType": "Concept",
        "additiveProductName": "String",
        "caloricDensity": "Decimal",
        "routeofAdministration": "Concept",
        "administration": "List<BackboneElement:NutritionOrder_Administration>",
        "maxVolumeToDeliver": "Decimal",
        "administrationInstruction": "String",
    },
    "NutritionOrder_Nutrient": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "modifier": "Concept",
        "amount": "Decimal",
    },
    "NutritionOrder_OralDiet": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "List<Concept>",
        "schedule": "List<BackboneElement:Timing>",
        "nutrient": "List<BackboneElement:NutritionOrder_Nutrient>",
        "texture": "List<BackboneElement:NutritionOrder_Texture>",
        "fluidConsistencyType": "List<Concept>",
        "instruction": "String",
    },
    "NutritionOrder_Supplement": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "productName": "String",
        "schedule": "List<BackboneElement:Timing>",
        "quantity": "Decimal",
        "instruction": "String",
    },
    "NutritionOrder_Texture": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "modifier": "Concept",
        "foodType": "Concept",
    },
    "ObservationDefinition_QualifiedInterval": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "category": "String",
        "range": "Interval<Decimal>",
        "context": "Concept",
        "appliesTo": "List<Concept>",
        "gender": "String",
        "age": "Interval<Decimal>",
        "gestationalAge": "Interval<Decimal>",
        "condition": "String",
    },
    "ObservationDefinition_QuantitativeDetails": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "customaryUnit": "Concept",
        "unit": "Concept",
        "conversionFactor": "Decimal",
        "decimalPrecision": "Integer",
    },
    "Observation_Component": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Concept",
        "value": "Choice:Interval<DateTime>",
        "dataAbsentReason": "Concept",
        "interpretation": "List<Concept>",
        "referenceRange": "List<BackboneElement:Observation_ReferenceRange>",
    },
    "Observation_ReferenceRange": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "low": "Decimal",
        "high": "Decimal",
        "type": "Concept",
        "appliesTo": "List<Concept>",
        "age": "Interval<Decimal>",
        "text": "String",
    },
    "OperationDefinition_Binding": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "strength": "String",
        "valueSet": "String",
    },
    "OperationDefinition_Overload": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "parameterName": "List<String>",
        "comment": "String",
    },
    "OperationDefinition_Parameter": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "name": "Code",
        "use": "String",
        "min": "Integer",
        "max": "String",
        "documentation": "String",
        "type": "Code",
        "targetProfile": "List<String>",
        "searchType": "String",
        "binding": "BackboneElement:OperationDefinition_Binding",
        "referencedFrom": "List<BackboneElement:OperationDefinition_ReferencedFrom>",
        "part": "List<BackboneElement:OperationDefinition_Parameter>",
    },
    "OperationDefinition_ReferencedFrom": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "source": "String",
        "sourceId": "String",
    },
    "OperationOutcome_Issue": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "severity": "String",
        "code": "String",
        "details": "Concept",
        "diagnostics": "String",
        "location": "List<String>",
        "expression": "List<String>",
    },
    "Organization_Contact": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "purpose": "Concept",
        "name": "BackboneElement:HumanName",
        "telecom": "List<BackboneElement:ContactPoint>",
        "address": "BackboneElement:Address",
    },
    "ParameterDefinition": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "name": "Code",
        "use": "Code",
        "min": "Integer",
        "max": "String",
        "documentation": "String",
        "type": "Code",
        "profile": "String",
    },
    "Parameters_Parameter": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "name": "String",
        "value": "Choice:Interval<DateTime>",
        "resource": "String",
        "part": "List<BackboneElement:Parameters_Parameter>",
    },
    "Patient_Communication": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "language": "Concept",
        "preferred": "Boolean",
    },
    "Patient_Contact": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "relationship": "List<Concept>",
        "name": "BackboneElement:HumanName",
        "telecom": "List<BackboneElement:ContactPoint>",
        "address": "BackboneElement:Address",
        "gender": "String",
        "organization": "BackboneElement:Reference",
        "period": "Interval<DateTime>",
    },
    "Patient_Link": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "other": "BackboneElement:Reference",
        "type": "String",
    },
    "PaymentReconciliation_Detail": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "BackboneElement:Identifier",
        "predecessor": "BackboneElement:Identifier",
        "type": "Concept",
        "request": "BackboneElement:Reference",
        "submitter": "BackboneElement:Reference",
        "response": "BackboneElement:Reference",
        "date": "Date",
        "responsible": "BackboneElement:Reference",
        "payee": "BackboneElement:Reference",
        "amount": "BackboneElement:Money",
    },
    "PaymentReconciliation_ProcessNote": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "String",
        "text": "String",
    },
    "Period": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "start": "DateTime",
        "end": "DateTime",
    },
    "Person_Link": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "target": "BackboneElement:Reference",
        "assurance": "String",
    },
    "PlanDefinition_Action": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "prefix": "String",
        "title": "String",
        "description": "String",
        "textEquivalent": "String",
        "priority": "Code",
        "code": "List<Concept>",
        "reason": "List<Concept>",
        "documentation": "List<BackboneElement:RelatedArtifact>",
        "goalId": "List<String>",
        "subject": "Choice:String",
        "trigger": "List<BackboneElement:TriggerDefinition>",
        "condition": "List<BackboneElement:PlanDefinition_Condition>",
        "input": "List<BackboneElement:DataRequirement>",
        "output": "List<BackboneElement:DataRequirement>",
        "relatedAction": "List<BackboneElement:PlanDefinition_RelatedAction>",
        "timing": "Choice:Interval<DateTime>",
        "participant": "List<BackboneElement:PlanDefinition_Participant>",
        "type": "Concept",
        "groupingBehavior": "String",
        "selectionBehavior": "String",
        "requiredBehavior": "String",
        "precheckBehavior": "String",
        "cardinalityBehavior": "String",
        "definition": "Choice:String",
        "transform": "String",
        "dynamicValue": "List<BackboneElement:PlanDefinition_DynamicValue>",
        "action": "List<BackboneElement:PlanDefinition_Action>",
    },
    "PlanDefinition_Condition": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "kind": "String",
        "expression": "BackboneElement:Expression",
    },
    "PlanDefinition_DynamicValue": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "path": "String",
        "expression": "BackboneElement:Expression",
    },
    "PlanDefinition_Goal": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "category": "Concept",
        "description": "Concept",
        "priority": "Concept",
        "start": "Concept",
        "addresses": "List<Concept>",
        "documentation": "List<BackboneElement:RelatedArtifact>",
        "target": "List<BackboneElement:PlanDefinition_Target>",
    },
    "PlanDefinition_Participant": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "String",
        "role": "Concept",
    },
    "PlanDefinition_RelatedAction": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "actionId": "String",
        "relationship": "String",
        "offset": "Choice:String",
    },
    "PlanDefinition_Target": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "measure": "Concept",
        "detail": "Choice:String",
        "due": "BackboneElement:Duration",
    },
    "Population": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "age": "Choice:String",
        "gender": "Concept",
        "race": "Concept",
        "physiologicalCondition": "Concept",
    },
    "PractitionerRole_AvailableTime": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "daysOfWeek": "List<Code>",
        "allDay": "Boolean",
        "availableStartTime": "String",
        "availableEndTime": "String",
    },
    "PractitionerRole_NotAvailable": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "description": "String",
        "during": "Interval<DateTime>",
    },
    "Practitioner_Qualification": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "code": "Concept",
        "period": "Interval<DateTime>",
        "issuer": "BackboneElement:Reference",
    },
    "Procedure_FocalDevice": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "action": "Concept",
        "manipulated": "BackboneElement:Reference",
    },
    "Procedure_Performer": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "function": "Concept",
        "actor": "BackboneElement:Reference",
        "onBehalfOf": "BackboneElement:Reference",
    },
    "ProdCharacteristic": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "height": "Decimal",
        "width": "Decimal",
        "depth": "Decimal",
        "weight": "Decimal",
        "nominalVolume": "Decimal",
        "externalDiameter": "Decimal",
        "shape": "String",
        "color": "List<String>",
        "imprint": "List<String>",
        "image": "List<BackboneElement:Attachment>",
        "scoring": "Concept",
    },
    "ProductShelfLife": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "BackboneElement:Identifier",
        "type": "Concept",
        "period": "Decimal",
        "specialPrecautionsForStorage": "List<Concept>",
    },
    "Provenance_Agent": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "role": "List<Concept>",
        "who": "BackboneElement:Reference",
        "onBehalfOf": "BackboneElement:Reference",
    },
    "Provenance_Entity": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "role": "String",
        "what": "BackboneElement:Reference",
        "agent": "List<BackboneElement:Provenance_Agent>",
    },
    "Quantity": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "value": "Decimal",
        "comparator": "String",
        "unit": "String",
        "system": "String",
        "code": "Code",
    },
    "QuestionnaireResponse_Answer": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "value": "Choice:String",
        "item": "List<BackboneElement:QuestionnaireResponse_Item>",
    },
    "QuestionnaireResponse_Item": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "linkId": "String",
        "definition": "String",
        "text": "String",
        "answer": "List<BackboneElement:QuestionnaireResponse_Answer>",
        "item": "List<BackboneElement:QuestionnaireResponse_Item>",
    },
    "Questionnaire_AnswerOption": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "value": "Choice:String",
        "initialSelected": "Boolean",
    },
    "Questionnaire_EnableWhen": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "question": "String",
        "operator": "String",
        "answer": "Choice:String",
    },
    "Questionnaire_Initial": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "value": "Choice:String",
    },
    "Questionnaire_Item": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "linkId": "String",
        "definition": "String",
        "code": "List<Code>",
        "prefix": "String",
        "text": "String",
        "type": "String",
        "enableWhen": "List<BackboneElement:Questionnaire_EnableWhen>",
        "enableBehavior": "String",
        "required": "Boolean",
        "repeats": "Boolean",
        "readOnly": "Boolean",
        "maxLength": "Integer",
        "answerValueSet": "String",
        "answerOption": "List<BackboneElement:Questionnaire_AnswerOption>",
        "initial": "List<BackboneElement:Questionnaire_Initial>",
        "item": "List<BackboneElement:Questionnaire_Item>",
    },
    "Range": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "low": "Decimal",
        "high": "Decimal",
    },
    "Ratio": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "numerator": "Decimal",
        "denominator": "Decimal",
    },
    "Reference": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "reference": "String",
        "type": "String",
        "identifier": "BackboneElement:Identifier",
        "display": "String",
    },
    "RelatedArtifact": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "type": "String",
        "label": "String",
        "display": "String",
        "citation": "String",
        "url": "String",
        "document": "BackboneElement:Attachment",
        "resource": "String",
    },
    "RelatedPerson_Communication": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "language": "Concept",
        "preferred": "Boolean",
    },
    "RequestGroup_Action": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "prefix": "String",
        "title": "String",
        "description": "String",
        "textEquivalent": "String",
        "priority": "Code",
        "code": "List<Concept>",
        "documentation": "List<BackboneElement:RelatedArtifact>",
        "condition": "List<BackboneElement:RequestGroup_Condition>",
        "relatedAction": "List<BackboneElement:RequestGroup_RelatedAction>",
        "timing": "Choice:Interval<DateTime>",
        "participant": "List<BackboneElement:Reference>",
        "type": "Concept",
        "groupingBehavior": "Code",
        "selectionBehavior": "Code",
        "requiredBehavior": "Code",
        "precheckBehavior": "Code",
        "cardinalityBehavior": "Code",
        "resource": "BackboneElement:Reference",
        "action": "List<BackboneElement:RequestGroup_Action>",
    },
    "RequestGroup_Condition": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "kind": "Code",
        "expression": "BackboneElement:Expression",
    },
    "RequestGroup_RelatedAction": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "actionId": "String",
        "relationship": "Code",
        "offset": "Choice:String",
    },
    "ResearchElementDefinition_Characteristic": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "definition": "Choice:String",
        "usageContext": "List<BackboneElement:UsageContext>",
        "exclude": "Boolean",
        "unitOfMeasure": "Concept",
        "studyEffectiveDescription": "String",
        "": "Choice:Interval<DateTime>",
        "studyEffectiveTimeFromStart": "BackboneElement:Duration",
        "studyEffectiveGroupMeasure": "String",
        "participantEffectiveDescription": "String",
        "participantEffectiveTimeFromStart": "BackboneElement:Duration",
        "participantEffectiveGroupMeasure": "String",
    },
    "ResearchStudy_Arm": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "name": "String",
        "type": "Concept",
        "description": "String",
    },
    "ResearchStudy_Objective": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "name": "String",
        "type": "Concept",
    },
    "RiskAssessment_Prediction": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "outcome": "Concept",
        "probability": "Choice:String",
        "qualitativeRisk": "Concept",
        "relativeRisk": "Decimal",
        "when": "Choice:Interval<DateTime>",
        "rationale": "String",
    },
    "RiskEvidenceSynthesis_Certainty": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "rating": "List<Concept>",
        "note": "List<BackboneElement:Annotation>",
        "certaintySubcomponent": "List<BackboneElement:RiskEvidenceSynthesis_CertaintySubcomponent>",
    },
    "RiskEvidenceSynthesis_CertaintySubcomponent": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "rating": "List<Concept>",
        "note": "List<BackboneElement:Annotation>",
    },
    "RiskEvidenceSynthesis_PrecisionEstimate": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "level": "Decimal",
        "from": "Decimal",
        "to": "Decimal",
    },
    "RiskEvidenceSynthesis_RiskEstimate": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "description": "String",
        "type": "Concept",
        "value": "Decimal",
        "unitOfMeasure": "Concept",
        "denominatorCount": "Integer",
        "numeratorCount": "Integer",
        "precisionEstimate": "List<BackboneElement:RiskEvidenceSynthesis_PrecisionEstimate>",
    },
    "RiskEvidenceSynthesis_SampleSize": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "description": "String",
        "numberOfStudies": "Integer",
        "numberOfParticipants": "Integer",
    },
    "SampledData": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "origin": "Decimal",
        "period": "Decimal",
        "factor": "Decimal",
        "lowerLimit": "Decimal",
        "upperLimit": "Decimal",
        "dimensions": "String",
        "data": "String",
    },
    "SearchParameter_Component": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "definition": "String",
        "expression": "String",
    },
    "Signature": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "type": "List<Code>",
        "when": "String",
        "who": "BackboneElement:Reference",
        "onBehalfOf": "BackboneElement:Reference",
        "targetFormat": "Code",
        "sigFormat": "Code",
        "data": "String",
    },
    "SpecimenDefinition_Additive": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "additive": "Choice:String",
    },
    "SpecimenDefinition_Container": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "material": "Concept",
        "type": "Concept",
        "cap": "Concept",
        "description": "String",
        "capacity": "Decimal",
        "minimumVolume": "Choice:String",
        "additive": "List<BackboneElement:SpecimenDefinition_Additive>",
        "preparation": "String",
    },
    "SpecimenDefinition_Handling": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "temperatureQualifier": "Concept",
        "temperatureRange": "Interval<Decimal>",
        "maxDuration": "BackboneElement:Duration",
        "instruction": "String",
    },
    "SpecimenDefinition_TypeTested": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "isDerived": "Boolean",
        "type": "Concept",
        "preference": "String",
        "container": "BackboneElement:SpecimenDefinition_Container",
        "requirement": "String",
        "retentionTime": "BackboneElement:Duration",
        "rejectionCriterion": "List<Concept>",
        "handling": "List<BackboneElement:SpecimenDefinition_Handling>",
    },
    "Specimen_Collection": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "collector": "BackboneElement:Reference",
        "collected": "Choice:Interval<DateTime>",
        "duration": "BackboneElement:Duration",
        "quantity": "Decimal",
        "method": "Concept",
        "bodySite": "Concept",
        "fastingStatus": "Choice:String",
    },
    "Specimen_Container": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "List<BackboneElement:Identifier>",
        "description": "String",
        "type": "Concept",
        "capacity": "Decimal",
        "specimenQuantity": "Decimal",
        "additive": "Choice:String",
    },
    "Specimen_Processing": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "description": "String",
        "procedure": "Concept",
        "additive": "List<BackboneElement:Reference>",
        "time": "Choice:Interval<DateTime>",
    },
    "StructureDefinition_Context": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "String",
        "expression": "String",
    },
    "StructureDefinition_Differential": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "element": "List<BackboneElement:ElementDefinition>",
    },
    "StructureDefinition_Mapping": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identity": "String",
        "uri": "String",
        "name": "String",
        "comment": "String",
    },
    "StructureDefinition_Snapshot": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "element": "List<BackboneElement:ElementDefinition>",
    },
    "StructureMap_Dependent": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "name": "String",
        "variable": "List<String>",
    },
    "StructureMap_Group": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "name": "String",
        "extends": "String",
        "typeMode": "String",
        "documentation": "String",
        "input": "List<BackboneElement:StructureMap_Input>",
        "rule": "List<BackboneElement:StructureMap_Rule>",
    },
    "StructureMap_Input": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "name": "String",
        "type": "String",
        "mode": "String",
        "documentation": "String",
    },
    "StructureMap_Parameter": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "value": "Choice:String",
    },
    "StructureMap_Rule": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "name": "String",
        "source": "List<BackboneElement:StructureMap_Source>",
        "target": "List<BackboneElement:StructureMap_Target>",
        "rule": "List<BackboneElement:StructureMap_Rule>",
        "dependent": "List<BackboneElement:StructureMap_Dependent>",
        "documentation": "String",
    },
    "StructureMap_Source": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "context": "String",
        "min": "Integer",
        "max": "String",
        "type": "String",
        "defaultValue": "Choice:Interval<DateTime>",
        "element": "String",
        "listMode": "String",
        "variable": "String",
        "condition": "String",
        "check": "String",
        "logMessage": "String",
    },
    "StructureMap_Structure": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "url": "String",
        "mode": "String",
        "alias": "String",
        "documentation": "String",
    },
    "StructureMap_Target": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "context": "String",
        "contextType": "String",
        "element": "String",
        "variable": "String",
        "listMode": "List<String>",
        "listRuleId": "String",
        "transform": "String",
        "parameter": "List<BackboneElement:StructureMap_Parameter>",
    },
    "Subscription_Channel": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "String",
        "endpoint": "String",
        "payload": "Code",
        "header": "List<String>",
    },
    "SubstanceAmount": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "amount": "Choice:String",
        "amountType": "Concept",
        "amountText": "String",
        "referenceRange": "BackboneElement:SubstanceAmount_ReferenceRange",
    },
    "SubstanceAmount_ReferenceRange": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "lowLimit": "Decimal",
        "highLimit": "Decimal",
    },
    "SubstanceNucleicAcid_Linkage": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "connectivity": "String",
        "identifier": "BackboneElement:Identifier",
        "name": "String",
        "residueSite": "String",
    },
    "SubstanceNucleicAcid_Subunit": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "subunit": "Integer",
        "sequence": "String",
        "length": "Integer",
        "sequenceAttachment": "BackboneElement:Attachment",
        "fivePrime": "Concept",
        "threePrime": "Concept",
        "linkage": "List<BackboneElement:SubstanceNucleicAcid_Linkage>",
        "sugar": "List<BackboneElement:SubstanceNucleicAcid_Sugar>",
    },
    "SubstanceNucleicAcid_Sugar": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "BackboneElement:Identifier",
        "name": "String",
        "residueSite": "String",
    },
    "SubstancePolymer_DegreeOfPolymerisation": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "": "Choice:String",
    },
    "SubstancePolymer_MonomerSet": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "": "Choice:String",
    },
    "SubstancePolymer_Repeat": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "": "Choice:String",
    },
    "SubstancePolymer_RepeatUnit": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "": "Choice:String",
    },
    "SubstancePolymer_StartingMaterial": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "": "Choice:String",
    },
    "SubstancePolymer_StructuralRepresentation": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "": "Choice:String",
    },
    "SubstanceProtein_Subunit": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "subunit": "Integer",
        "sequence": "Choice:String",
        "length": "Integer",
        "": "Choice:String",
        "nTerminalModification": "String",
        "cTerminalModification": "String",
    },
    "SubstanceReferenceInformation_Classification": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "": "Choice:String",
    },
    "SubstanceReferenceInformation_Gene": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "": "Choice:String",
    },
    "SubstanceReferenceInformation_GeneElement": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "": "Choice:String",
    },
    "SubstanceReferenceInformation_Target": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "": "Choice:String",
    },
    "SubstanceSourceMaterial_Author": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "authorType": "Concept",
        "authorDescription": "String",
    },
    "SubstanceSourceMaterial_FractionDescription": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "fraction": "String",
        "materialType": "Concept",
    },
    "SubstanceSourceMaterial_Hybrid": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "maternalOrganismId": "String",
        "maternalOrganismName": "String",
        "paternalOrganismId": "String",
        "paternalOrganismName": "String",
        "hybridType": "Concept",
    },
    "SubstanceSourceMaterial_Organism": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "family": "Concept",
        "genus": "Concept",
        "species": "Concept",
        "intraspecificType": "Concept",
        "intraspecificDescription": "String",
        "author": "List<BackboneElement:SubstanceSourceMaterial_Author>",
        "hybrid": "BackboneElement:SubstanceSourceMaterial_Hybrid",
        "organismGeneral": "BackboneElement:SubstanceSourceMaterial_OrganismGeneral",
    },
    "SubstanceSourceMaterial_OrganismGeneral": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "kingdom": "Concept",
        "phylum": "Concept",
        "class": "Concept",
        "order": "Concept",
    },
    "SubstanceSourceMaterial_PartDescription": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "part": "Concept",
        "partLocation": "Concept",
    },
    "SubstanceSpecification_Code": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Concept",
        "status": "Concept",
        "statusDate": "DateTime",
        "comment": "String",
        "source": "List<BackboneElement:Reference>",
    },
    "SubstanceSpecification_Isotope": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "BackboneElement:Identifier",
        "name": "Concept",
        "substitution": "Concept",
        "halfLife": "Decimal",
        "molecularWeight": "BackboneElement:SubstanceSpecification_MolecularWeight",
    },
    "SubstanceSpecification_Moiety": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "role": "Concept",
        "identifier": "BackboneElement:Identifier",
        "name": "String",
        "stereochemistry": "Concept",
        "opticalActivity": "Concept",
        "molecularFormula": "String",
        "amount": "Choice:String",
    },
    "SubstanceSpecification_MolecularWeight": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "method": "Concept",
        "type": "Concept",
        "amount": "Decimal",
    },
    "SubstanceSpecification_Name": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "name": "String",
        "type": "Concept",
        "status": "Concept",
        "preferred": "Boolean",
        "language": "List<Concept>",
        "domain": "List<Concept>",
        "jurisdiction": "List<Concept>",
        "synonym": "List<BackboneElement:SubstanceSpecification_Name>",
        "translation": "List<BackboneElement:SubstanceSpecification_Name>",
        "official": "List<BackboneElement:SubstanceSpecification_Official>",
        "source": "List<BackboneElement:Reference>",
    },
    "SubstanceSpecification_Official": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "authority": "Concept",
        "status": "Concept",
        "date": "DateTime",
    },
    "SubstanceSpecification_Property": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "category": "Concept",
        "code": "Concept",
        "parameters": "String",
        "definingSubstance": "Choice:String",
        "amount": "Choice:String",
    },
    "SubstanceSpecification_Relationship": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "substance": "Choice:String",
        "relationship": "Concept",
        "isDefining": "Boolean",
        "amount": "Choice:String",
        "amountRatioLowLimit": "BackboneElement:Ratio",
        "amountType": "Concept",
        "source": "List<BackboneElement:Reference>",
    },
    "SubstanceSpecification_Representation": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "representation": "String",
        "attachment": "BackboneElement:Attachment",
    },
    "SubstanceSpecification_Structure": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "stereochemistry": "Concept",
        "opticalActivity": "Concept",
        "molecularFormula": "String",
        "molecularFormulaByMoiety": "String",
        "isotope": "List<BackboneElement:SubstanceSpecification_Isotope>",
        "molecularWeight": "BackboneElement:SubstanceSpecification_MolecularWeight",
        "source": "List<BackboneElement:Reference>",
        "representation": "List<BackboneElement:SubstanceSpecification_Representation>",
    },
    "Substance_Ingredient": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "quantity": "BackboneElement:Ratio",
        "substance": "Choice:String",
    },
    "Substance_Instance": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "BackboneElement:Identifier",
        "expiry": "DateTime",
        "quantity": "Decimal",
    },
    "SupplyDelivery_SuppliedItem": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "quantity": "Decimal",
        "item": "Choice:String",
    },
    "SupplyRequest_Parameter": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Concept",
        "value": "Choice:String",
    },
    "Task_Input": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "value": "Choice:Interval<DateTime>",
    },
    "Task_Output": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Concept",
        "value": "Choice:Interval<DateTime>",
    },
    "Task_Restriction": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "repetitions": "String",
        "period": "Interval<DateTime>",
        "recipient": "List<BackboneElement:Reference>",
    },
    "TerminologyCapabilities_Closure": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "translation": "Boolean",
    },
    "TerminologyCapabilities_CodeSystem": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "uri": "String",
        "version": "List<BackboneElement:TerminologyCapabilities_Version>",
        "subsumption": "Boolean",
    },
    "TerminologyCapabilities_Expansion": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "hierarchical": "Boolean",
        "paging": "Boolean",
        "incomplete": "Boolean",
        "parameter": "List<BackboneElement:TerminologyCapabilities_Parameter>",
        "textFilter": "String",
    },
    "TerminologyCapabilities_Filter": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Code",
        "op": "List<Code>",
    },
    "TerminologyCapabilities_Implementation": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "description": "String",
        "url": "String",
    },
    "TerminologyCapabilities_Parameter": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "name": "Code",
        "documentation": "String",
    },
    "TerminologyCapabilities_Software": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "name": "String",
        "version": "String",
    },
    "TerminologyCapabilities_Translation": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "needsMap": "Boolean",
    },
    "TerminologyCapabilities_ValidateCode": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "translations": "Boolean",
    },
    "TerminologyCapabilities_Version": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "String",
        "isDefault": "Boolean",
        "compositional": "Boolean",
        "language": "List<Code>",
        "filter": "List<BackboneElement:TerminologyCapabilities_Filter>",
        "property": "List<Code>",
    },
    "TestReport_Action": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "operation": "BackboneElement:TestReport_Operation",
        "assert": "BackboneElement:TestReport_Assert",
    },
    "TestReport_Action1": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "operation": "BackboneElement:TestReport_Operation",
        "assert": "BackboneElement:TestReport_Assert",
    },
    "TestReport_Action2": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "operation": "BackboneElement:TestReport_Operation",
    },
    "TestReport_Assert": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "result": "String",
        "message": "String",
        "detail": "String",
    },
    "TestReport_Operation": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "result": "String",
        "message": "String",
        "detail": "String",
    },
    "TestReport_Participant": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "String",
        "uri": "String",
        "display": "String",
    },
    "TestReport_Setup": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "action": "List<BackboneElement:TestReport_Action>",
    },
    "TestReport_Teardown": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "action": "List<BackboneElement:TestReport_Action2>",
    },
    "TestReport_Test": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "name": "String",
        "description": "String",
        "action": "List<BackboneElement:TestReport_Action1>",
    },
    "TestScript_Action": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "operation": "BackboneElement:TestScript_Operation",
        "assert": "BackboneElement:TestScript_Assert",
    },
    "TestScript_Action1": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "operation": "BackboneElement:TestScript_Operation",
        "assert": "BackboneElement:TestScript_Assert",
    },
    "TestScript_Action2": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "operation": "BackboneElement:TestScript_Operation",
    },
    "TestScript_Assert": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "label": "String",
        "description": "String",
        "direction": "String",
        "compareToSourceId": "String",
        "compareToSourceExpression": "String",
        "compareToSourcePath": "String",
        "contentType": "Code",
        "expression": "String",
        "headerField": "String",
        "minimumId": "String",
        "navigationLinks": "Boolean",
        "operator": "String",
        "path": "String",
        "requestMethod": "String",
        "requestURL": "String",
        "resource": "Code",
        "response": "String",
        "responseCode": "String",
        "sourceId": "String",
        "validateProfileId": "String",
        "value": "String",
        "warningOnly": "Boolean",
    },
    "TestScript_Capability": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "required": "Boolean",
        "validated": "Boolean",
        "description": "String",
        "origin": "List<Integer>",
        "destination": "Integer",
        "link": "List<String>",
        "capabilities": "String",
    },
    "TestScript_Destination": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "index": "Integer",
        "profile": "Code",
    },
    "TestScript_Fixture": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "autocreate": "Boolean",
        "autodelete": "Boolean",
        "resource": "BackboneElement:Reference",
    },
    "TestScript_Link": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "url": "String",
        "description": "String",
    },
    "TestScript_Metadata": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "link": "List<BackboneElement:TestScript_Link>",
        "capability": "List<BackboneElement:TestScript_Capability>",
    },
    "TestScript_Operation": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "type": "Code",
        "resource": "Code",
        "label": "String",
        "description": "String",
        "accept": "Code",
        "contentType": "Code",
        "destination": "Integer",
        "encodeRequestUrl": "Boolean",
        "method": "String",
        "origin": "Integer",
        "params": "String",
        "requestHeader": "List<BackboneElement:TestScript_RequestHeader>",
        "requestId": "String",
        "responseId": "String",
        "sourceId": "String",
        "targetId": "String",
        "url": "String",
    },
    "TestScript_Origin": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "index": "Integer",
        "profile": "Code",
    },
    "TestScript_RequestHeader": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "field": "String",
        "value": "String",
    },
    "TestScript_Setup": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "action": "List<BackboneElement:TestScript_Action>",
    },
    "TestScript_Teardown": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "action": "List<BackboneElement:TestScript_Action2>",
    },
    "TestScript_Test": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "name": "String",
        "description": "String",
        "action": "List<BackboneElement:TestScript_Action1>",
    },
    "TestScript_Variable": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "name": "String",
        "defaultValue": "String",
        "description": "String",
        "expression": "String",
        "headerField": "String",
        "hint": "String",
        "path": "String",
        "sourceId": "String",
    },
    "Timing": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "event": "List<DateTime>",
        "repeat": "BackboneElement:Timing_Repeat",
        "code": "Concept",
    },
    "Timing_Repeat": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "bounds": "Choice:Interval<DateTime>",
        "count": "String",
        "countMax": "String",
        "duration": "Decimal",
        "durationMax": "Decimal",
        "durationUnit": "String",
        "frequency": "String",
        "frequencyMax": "String",
        "period": "Decimal",
        "periodMax": "Decimal",
        "periodUnit": "String",
        "dayOfWeek": "List<Code>",
        "timeOfDay": "List<String>",
        "when": "List<String>",
        "offset": "String",
    },
    "TriggerDefinition": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "type": "String",
        "name": "String",
        "timing": "Choice:String",
        "data": "List<BackboneElement:DataRequirement>",
        "condition": "BackboneElement:Expression",
    },
    "UsageContext": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "code": "Code",
        "value": "Choice:String",
    },
    "ValueSet_Compose": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "lockedDate": "Date",
        "inactive": "Boolean",
        "include": "List<BackboneElement:ValueSet_Include>",
        "exclude": "List<BackboneElement:ValueSet_Include>",
    },
    "ValueSet_Concept": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "code": "Code",
        "display": "String",
        "designation": "List<BackboneElement:ValueSet_Designation>",
    },
    "ValueSet_Contains": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "system": "String",
        "abstract": "Boolean",
        "inactive": "Boolean",
        "version": "String",
        "code": "Code",
        "display": "String",
        "designation": "List<BackboneElement:ValueSet_Designation>",
        "contains": "List<BackboneElement:ValueSet_Contains>",
    },
    "ValueSet_Designation": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "language": "Code",
        "use": "Code",
        "value": "String",
    },
    "ValueSet_Expansion": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "identifier": "String",
        "timestamp": "DateTime",
        "total": "Integer",
        "offset": "Integer",
        "parameter": "List<BackboneElement:ValueSet_Parameter>",
        "contains": "List<BackboneElement:ValueSet_Contains>",
    },
    "ValueSet_Filter": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "property": "Code",
        "op": "String",
        "value": "String",
    },
    "ValueSet_Include": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "system": "String",
        "version": "String",
        "concept": "List<BackboneElement:ValueSet_Concept>",
        "filter": "List<BackboneElement:ValueSet_Filter>",
        "valueSet": "List<String>",
    },
    "ValueSet_Parameter": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "name": "String",
        "value": "Choice:String",
    },
    "VerificationResult_Attestation": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "who": "BackboneElement:Reference",
        "onBehalfOf": "BackboneElement:Reference",
        "communicationMethod": "Concept",
        "date": "Date",
        "sourceIdentityCertificate": "String",
        "proxyIdentityCertificate": "String",
        "proxySignature": "BackboneElement:Signature",
        "sourceSignature": "BackboneElement:Signature",
    },
    "VerificationResult_PrimarySource": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "who": "BackboneElement:Reference",
        "type": "List<Concept>",
        "communicationMethod": "List<Concept>",
        "validationStatus": "Concept",
        "validationDate": "DateTime",
        "canPushUpdates": "Concept",
        "pushTypeAvailable": "List<Concept>",
    },
    "VerificationResult_Validator": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "organization": "BackboneElement:Reference",
        "identityCertificate": "String",
        "attestationSignature": "BackboneElement:Signature",
    },
    "VisionPrescription_LensSpecification": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "product": "Concept",
        "eye": "String",
        "sphere": "Decimal",
        "cylinder": "Decimal",
        "axis": "Integer",
        "prism": "List<BackboneElement:VisionPrescription_Prism>",
        "add": "Decimal",
        "power": "Decimal",
        "backCurve": "Decimal",
        "diameter": "Decimal",
        "duration": "Decimal",
        "color": "String",
        "brand": "String",
        "note": "List<BackboneElement:Annotation>",
    },
    "VisionPrescription_Prism": {
        "id": "String",
        "extension": "List<BackboneElement:Extension>",
        "modifierExtension": "List<BackboneElement:Extension>",
        "amount": "Decimal",
        "base": "String",
    },
}
